@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@ class CTriggerExpression {
 	 * An options array
 	 *
 	 * Supported options:
-	 *   'lldmacros' => true	low-level discovery macros can contain in trigger expression
+	 *   'lldmacros' => true	Enable low-level discovery macros usage in trigger expression.
 	 *
 	 * @var array
 	 */
@@ -69,6 +69,13 @@ class CTriggerExpression {
 	 */
 	public $expression;
 
+	/**
+	 * Object containing the results of parsing.
+	 *
+	 * @var CTriggerExprParserResult
+	 */
+	public $result;
+
 	/**
 	 * Current cursor position.
 	 *
@@ -125,6 +132,13 @@ class CTriggerExpression {
 	 */
 	protected $lld_macro_parser;
 
+	/**
+	 * Parser for LLD macros with functions.
+	 *
+	 * @var CLLDMacroFunctionParser
+	 */
+	protected $lld_macro_function_parser;
+
 	/**
 	 * Parser for user macros.
 	 *
@@ -139,19 +153,12 @@ class CTriggerExpression {
 	 */
 	protected $spaceChars = [' ' => true, "\r" => true, "\n" => true, "\t" => true];
 
-	/**
-	 * Object containing the results of parsing.
-	 *
-	 * @var CTriggerExpressionParserResult
-	 */
-	protected $result;
-
 	/**
 	 * @param array $options
-	 * @param bool $options['lldmacros']
+	 * @param bool  $options['lldmacros']
 	 */
-	public function __construct($options = []) {
-		if (isset($options['lldmacros'])) {
+	public function __construct(array $options = []) {
+		if (array_key_exists('lldmacros', $options)) {
 			$this->options['lldmacros'] = $options['lldmacros'];
 		}
 
@@ -162,6 +169,7 @@ class CTriggerExpression {
 		$this->function_macro_parser = new CFunctionMacroParser();
 		$this->function_parser = new CFunctionParser();
 		$this->lld_macro_parser = new CLLDMacroParser();
+		$this->lld_macro_function_parser = new CLLDMacroFunctionParser;
 		$this->user_macro_parser = new CUserMacroParser();
 	}
 
@@ -190,11 +198,11 @@ class CTriggerExpression {
 	 *
 	 * @param string $expression
 	 *
-	 * @return CTriggerExpressionParserResult|bool   returns a result object if a match has been found or false otherwise
+	 * @return CTriggerExprParserResult|bool   returns a result object if a match has been found or false otherwise
 	 */
 	public function parse($expression) {
 		// initializing local variables
-		$this->result = new CTriggerExpressionParserResult();
+		$this->result = new CTriggerExprParserResult();
 		$this->isValid = true;
 		$this->error = '';
 		$this->expressions = [];
@@ -220,20 +228,20 @@ class CTriggerExpression {
 					switch ($char) {
 						case '-':
 							$state = self::STATE_AFTER_MINUS_OPERATOR;
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_OPERATOR,
 								$char, $this->pos, 1
 							);
 							break;
 						case '(':
 							$state = self::STATE_AFTER_OPEN_BRACE;
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_OPEN_BRACE,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_OPEN_BRACE,
 								$char, $this->pos, 1
 							);
 							$level++;
 							break;
 						default:
 							if ($this->parseUsing($this->notOperatorParser,
-									CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR)) {
+									CTriggerExprParserResult::TOKEN_TYPE_OPERATOR)) {
 								$state = self::STATE_AFTER_NOT_OPERATOR;
 							}
 							elseif ($this->parseConstant()) {
@@ -249,13 +257,13 @@ class CTriggerExpression {
 					switch ($char) {
 						case '-':
 							$state = self::STATE_AFTER_MINUS_OPERATOR;
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_OPERATOR,
 								$char, $this->pos, 1
 							);
 							break;
 						case '(':
 							$state = self::STATE_AFTER_OPEN_BRACE;
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_OPEN_BRACE,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_OPEN_BRACE,
 								$char, $this->pos, 1
 							);
 							$level++;
@@ -271,7 +279,7 @@ class CTriggerExpression {
 							}
 
 							if ($this->parseUsing($this->notOperatorParser,
-									CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR)) {
+									CTriggerExprParserResult::TOKEN_TYPE_OPERATOR)) {
 
 								$state = self::STATE_AFTER_NOT_OPERATOR;
 							}
@@ -287,13 +295,13 @@ class CTriggerExpression {
 							if (!$afterSpace) {
 								break 3;
 							}
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_OPERATOR,
 								$char, $this->pos, 1
 							);
 							$state = self::STATE_AFTER_MINUS_OPERATOR;
 							break;
 						case '(':
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_OPEN_BRACE,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_OPEN_BRACE,
 								$char, $this->pos, 1
 							);
 							$state = self::STATE_AFTER_OPEN_BRACE;
@@ -305,7 +313,7 @@ class CTriggerExpression {
 							}
 
 							if ($this->parseUsing($this->notOperatorParser,
-									CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR)) {
+									CTriggerExprParserResult::TOKEN_TYPE_OPERATOR)) {
 
 								$state = self::STATE_AFTER_NOT_OPERATOR;
 							}
@@ -324,21 +332,21 @@ class CTriggerExpression {
 							if ($level == 0) {
 								break 3;
 							}
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_CLOSE_BRACE,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_CLOSE_BRACE,
 								$char, $this->pos, 1
 							);
 							$level--;
 							break;
 						default:
 							if ($this->parseUsing($this->binaryOperatorParser,
-									CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR)) {
+									CTriggerExprParserResult::TOKEN_TYPE_OPERATOR)) {
 
 								$state = self::STATE_AFTER_BINARY_OPERATOR;
 								break;
 							}
 
 							if ($this->parseUsing($this->logicalOperatorParser,
-									CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR)) {
+									CTriggerExprParserResult::TOKEN_TYPE_OPERATOR)) {
 
 								$state = self::STATE_AFTER_LOGICAL_OPERATOR;
 								break;
@@ -355,7 +363,7 @@ class CTriggerExpression {
 							if ($level == 0) {
 								break 3;
 							}
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_CLOSE_BRACE,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_CLOSE_BRACE,
 								$char, $this->pos, 1
 							);
 							$level--;
@@ -363,7 +371,7 @@ class CTriggerExpression {
 							break;
 						default:
 							if ($this->parseUsing($this->binaryOperatorParser,
-									CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR)) {
+									CTriggerExprParserResult::TOKEN_TYPE_OPERATOR)) {
 
 								$state = self::STATE_AFTER_BINARY_OPERATOR;
 								break;
@@ -374,7 +382,7 @@ class CTriggerExpression {
 							}
 
 							if ($this->parseUsing($this->logicalOperatorParser,
-									CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR)) {
+									CTriggerExprParserResult::TOKEN_TYPE_OPERATOR)) {
 
 								$state = self::STATE_AFTER_LOGICAL_OPERATOR;
 							}
@@ -390,13 +398,13 @@ class CTriggerExpression {
 							if (!$afterSpace) {
 								break 3;
 							}
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_OPERATOR,
 								$char, $this->pos, 1
 							);
 							$state = self::STATE_AFTER_MINUS_OPERATOR;
 							break;
 						case '(':
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_OPEN_BRACE,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_OPEN_BRACE,
 								$char, $this->pos, 1
 							);
 							$state = self::STATE_AFTER_OPEN_BRACE;
@@ -419,7 +427,7 @@ class CTriggerExpression {
 				case self::STATE_AFTER_MINUS_OPERATOR:
 					switch ($char) {
 						case '(':
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_OPEN_BRACE,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_OPEN_BRACE,
 								$char, $this->pos, 1
 							);
 							$state = self::STATE_AFTER_OPEN_BRACE;
@@ -499,27 +507,32 @@ class CTriggerExpression {
 
 	/**
 	 * Parses a constant in the trigger expression and moves a current position ($this->pos) on a last symbol of the
-	 * constant
+	 * constant.
 	 *
 	 * The constant can be:
 	 *  - trigger function like {host:item[].func()}
 	 *  - floating point number; can be with suffix [KMGTsmhdw]
 	 *  - macro like {TRIGGER.VALUE}
 	 *  - user macro like {$MACRO}
+	 *  - LLD macro like {#LLD}
+	 *  - LLD macro with function like {{#LLD}.func())}
 	 *
-	 * @return bool returns true if parsed successfully, false otherwise
+	 * @return bool  Returns true if parsed successfully, false otherwise.
 	 */
 	private function parseConstant() {
 		if ($this->parseFunctionMacro() || $this->parseNumber()
-				|| $this->parseUsing($this->user_macro_parser, CTriggerExpressionParserResult::TOKEN_TYPE_USER_MACRO)
-				|| $this->parseUsing($this->macro_parser, CTriggerExpressionParserResult::TOKEN_TYPE_MACRO)) {
+				|| $this->parseUsing($this->user_macro_parser, CTriggerExprParserResult::TOKEN_TYPE_USER_MACRO)
+				|| $this->parseUsing($this->macro_parser, CTriggerExprParserResult::TOKEN_TYPE_MACRO)) {
 			return true;
 		}
 
-		// LLD macro support for trigger prototypes
-		if ($this->options['lldmacros']
-				&& $this->parseUsing($this->lld_macro_parser, CTriggerExpressionParserResult::TOKEN_TYPE_LLD_MACRO)) {
-			return true;
+		// LLD macro support for trigger prototypes.
+		if ($this->options['lldmacros']) {
+			if ($this->parseUsing($this->lld_macro_parser, CTriggerExprParserResult::TOKEN_TYPE_LLD_MACRO)
+					|| $this->parseUsing($this->lld_macro_function_parser,
+							CTriggerExprParserResult::TOKEN_TYPE_LLD_MACRO)) {
+				return true;
+			}
 		}
 
 		return false;
@@ -550,7 +563,7 @@ class CTriggerExpression {
 			$function_param_list[] = $this->function_parser->getParam($n);
 		}
 
-		$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_FUNCTION_MACRO,
+		$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_FUNCTION_MACRO,
 			$this->function_macro_parser->getMatch(), $startPos, $this->function_macro_parser->getLength(),
 			[
 				'host' => $this->function_macro_parser->getHost(),
@@ -619,7 +632,7 @@ class CTriggerExpression {
 		$numberLength = $j - $this->pos;
 
 		$this->result->addToken(
-			CTriggerExpressionParserResult::TOKEN_TYPE_NUMBER,
+			CTriggerExprParserResult::TOKEN_TYPE_NUMBER,
 			substr($this->expression, $this->pos, $numberLength),
 			$this->pos,
 			$numberLength,
