@@ -2003,12 +2003,14 @@ function getMapHighligts(array $map, array $map_info) {
  *
  * @param array $sysmap
  * @param array $sysmap['show_suppressed']  Whether to show suppressed problems.
+ * @param array $sysmap['show_unack']       Property specified in sysmap's 'Problem display' field. Used to determine
+ *                                          whether to show unacknowledged problems only.
  * @param array $options                    Options used to retrieve actions.
  * @param int   $options['severity_min']    Minimal severity used.
  *
  * @return array
  */
-function getMapLinktriggerInfo($sysmap, $options) {
+function getMapLinkTriggerInfo($sysmap, $options) {
 	if (!array_key_exists('severity_min', $options)) {
 		$options['severity_min'] = TRIGGER_SEVERITY_NOT_CLASSIFIED;
 	}
@@ -2017,18 +2019,23 @@ function getMapLinktriggerInfo($sysmap, $options) {
 
 	foreach ($sysmap['links'] as $link) {
 		foreach ($link['linktriggers'] as $linktrigger) {
-			$triggerids[$linktrigger['triggerid']] = $linktrigger['triggerid'];
+			$triggerids[$linktrigger['triggerid']] = true;
 		}
 	}
 
 	$trigger_options = [
 		'output' => ['status', 'value', 'priority'],
-		'triggerids' => $triggerids,
+		'triggerids' => array_keys($triggerids),
 		'monitored' => true,
 		'preservekeys' => true
 	];
 
-	return getTriggersWithActualSeverity($trigger_options, ['show_suppressed' => $sysmap['show_suppressed']]);
+	$problem_options = [
+		'show_suppressed' => $sysmap['show_suppressed'],
+		'acknowledged' => ($sysmap['show_unack'] == EXTACK_OPTION_UNACK) ? false : null
+	];
+
+	return getTriggersWithActualSeverity($trigger_options, $problem_options);
 }
 
 /**
