@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,6 +21,16 @@
 #include "comms.h"
 #include "log.h"
 #include "../zbxcrypto/tls_tcp.h"
+#include "zbxcompress.h"
+
+#ifdef _WINDOWS
+#	ifndef _WIN32_WINNT_WIN7
+#		define _WIN32_WINNT_WIN7		0x0601	/* allow compilation on older Windows systems */
+#	endif
+#	ifndef WSA_FLAG_NO_HANDLE_INHERIT
+#		define WSA_FLAG_NO_HANDLE_INHERIT	0x80	/* allow compilation on older Windows systems */
+#	endif
+#endif
 
 #define IPV4_MAX_CIDR_PREFIX	32	/* max number of bits in IPv4 CIDR prefix */
 #define IPV6_MAX_CIDR_PREFIX	128	/* max number of bits in IPv6 CIDR prefix */
@@ -61,12 +71,8 @@ const char	*zbx_socket_strerror(void)
 	return zbx_socket_strerror_message;
 }
 
-#ifdef HAVE___VA_ARGS__
-#	define zbx_set_socket_strerror(fmt, ...) __zbx_zbx_set_socket_strerror(ZBX_CONST_STRING(fmt), ##__VA_ARGS__)
-#else
-#	define zbx_set_socket_strerror __zbx_zbx_set_socket_strerror
-#endif
-static void	__zbx_zbx_set_socket_strerror(const char *fmt, ...)
+__zbx_attr_format_printf(1, 2)
+static void	zbx_set_socket_strerror(const char *fmt, ...)
 {
 	va_list args;
 
@@ -179,39 +185,67 @@ void	zbx_gethost_by_ip(const char *ip, char *host, size_t hostlen)
 #endif	/* HAVE_IPV6 */
 #endif	/* _WINDOWS */
 
+#ifdef _WINDOWS
+/******************************************************************************
+ *                                                                            *
+ * Function: zbx_is_win_ver_or_greater                                        *
+ *                                                                            *
+ * Purpose: check Windows version                                             *
+ *                                                                            *
+ * Parameters: major    - [IN] major windows version                          *
+ *             minor    - [IN] minor windows version                          *
+ *             servpack - [IN] service pack version                           *
+ *                                                                            *
+ * Return value: SUCCEED - Windows version matches input parameters           *
+ *                         or greater                                         *
+ *               FAIL    - Windows version is older                           *
+ *                                                                            *
+ * Comments: This is reimplementation of IsWindowsVersionOrGreater() from     *
+ *           Version Helper API. We need it because the original function is  *
+ *           only available in newer Windows toolchains (VS2013+)             *
+ *                                                                            *
+ ******************************************************************************/
+static int zbx_is_win_ver_or_greater(zbx_uint32_t major, zbx_uint32_t minor, zbx_uint32_t servpack)
+{
+	OSVERSIONINFOEXW vi = { sizeof(vi), major, minor, 0, 0, { 0 }, servpack, 0 };
+
+	/* no need to test for an error, check VersionHelpers.h and usage examples */
+
+	return VerifyVersionInfoW(&vi, VER_MAJORVERSION | VER_MINORVERSION | VER_SERVICEPACKMAJOR,
+			VerSetConditionMask(VerSetConditionMask(VerSetConditionMask(0,
+			VER_MAJORVERSION, VER_GREATER_EQUAL),
+			VER_MINORVERSION, VER_GREATER_EQUAL),
+			VER_SERVICEPACKMAJOR, VER_GREATER_EQUAL)) ? SUCCEED : FAIL;
+}
+#endif
+
 /******************************************************************************
  *                                                                            *
  * Function: zbx_socket_start                                                 *
  *                                                                            *
  * Purpose: Initialize Windows Sockets APIs                                   *
  *                                                                            *
+ * Parameters: error - [OUT] the error message                                *
+ *                                                                            *
  * Return value: SUCCEED or FAIL - an error occurred                          *
  *                                                                            *
  * Author: Eugene Grigorjev                                                   *
  *                                                                            *
  ******************************************************************************/
 #ifdef _WINDOWS
-
-#define ZBX_SOCKET_START()	if (FAIL == socket_started) socket_started = zbx_socket_start()
-
-int	socket_started = FAIL;	/* winXX threads require socket_started not to be static */
-
-static int	zbx_socket_start()
+int	zbx_socket_start(char **error)
 {
 	WSADATA	sockInfo;
 	int	ret;
 
 	if (0 != (ret = WSAStartup(MAKEWORD(2, 2), &sockInfo)))
 	{
-		zabbix_log(LOG_LEVEL_WARNING, "WSAStartup() failed: %s", strerror_from_system(ret));
+		*error = zbx_dsprintf(*error, "Cannot initialize Winsock DLL: %s", strerror_from_system(ret));
 		return FAIL;
 	}
 
 	return SUCCEED;
 }
-
-#else
-#	define ZBX_SOCKET_START()
 #endif
 
 /******************************************************************************
@@ -455,8 +489,6 @@ static int	zbx_socket_create(zbx_socket_t *s, int type, const char *source_ip, c
 		return FAIL;
 	}
 #endif
-	ZBX_SOCKET_START();
-
 	zbx_socket_clean(s);
 
 	zbx_snprintf(service, sizeof(service), "%hu", port);
@@ -497,7 +529,7 @@ static int	zbx_socket_create(zbx_socket_t *s, int type, const char *source_ip, c
 			goto out;
 		}
 
-		if (ZBX_PROTO_ERROR == bind(s->socket, ai_bind->ai_addr, ai_bind->ai_addrlen))
+		if (ZBX_PROTO_ERROR == zbx_bind(s->socket, ai_bind->ai_addr, ai_bind->ai_addrlen))
 		{
 			zbx_set_socket_strerror("bind() failed: %s", strerror_from_system(zbx_socket_last_error()));
 			func_socket_close(s);
@@ -505,7 +537,7 @@ static int	zbx_socket_create(zbx_socket_t *s, int type, const char *source_ip, c
 		}
 	}
 
-	if (SUCCEED != zbx_socket_connect(s, ai->ai_addr, ai->ai_addrlen, timeout, &error))
+	if (SUCCEED != zbx_socket_connect(s, ai->ai_addr, (socklen_t)ai->ai_addrlen, timeout, &error))
 	{
 		func_socket_close(s);
 		zbx_set_socket_strerror("cannot connect to [[%s]:%hu]: %s", ip, port, error);
@@ -565,8 +597,6 @@ static int	zbx_socket_create(zbx_socket_t *s, int type, const char *source_ip, c
 		return FAIL;
 	}
 #endif
-	ZBX_SOCKET_START();
-
 	zbx_socket_clean(s);
 
 	if (NULL == (hp = gethostbyname(ip)))
@@ -731,18 +761,18 @@ static ssize_t	zbx_tcp_write(zbx_socket_t *s, const char *buf, size_t len)
  ******************************************************************************/
 
 #define ZBX_TCP_HEADER_DATA	"ZBXD"
-#define ZBX_TCP_HEADER_VERSION	"\1"
 #define ZBX_TCP_HEADER		ZBX_TCP_HEADER_DATA ZBX_TCP_HEADER_VERSION
-#define ZBX_TCP_HEADER_LEN	5
+#define ZBX_TCP_HEADER_LEN	ZBX_CONST_STRLEN(ZBX_TCP_HEADER_DATA)
 
 int	zbx_tcp_send_ext(zbx_socket_t *s, const char *data, size_t len, unsigned char flags, int timeout)
 {
 #define ZBX_TLS_MAX_REC_LEN	16384
 
-	zbx_uint64_t	len64_le;
-	ssize_t		bytes_sent = 0, written = 0;
-	size_t		send_bytes;
+	ssize_t		bytes_sent, written = 0;
+	size_t		send_bytes, offset, send_len = len, reserved = 0;
 	int		ret = SUCCEED;
+	char		*compressed_data = NULL;
+	zbx_uint32_t	len32_le;
 
 	if (0 != timeout)
 		zbx_socket_timeout_set(s, timeout);
@@ -754,15 +784,36 @@ int	zbx_tcp_send_ext(zbx_socket_t *s, const char *data, size_t len, unsigned cha
 								/* will be short-lived in CPU cache. Static buffer is */
 								/* not used on purpose.				      */
 
-		memcpy(header_buf, ZBX_TCP_HEADER, (size_t)ZBX_TCP_HEADER_LEN);
+		if (0 != (flags & ZBX_TCP_COMPRESS))
+		{
+			if (SUCCEED != zbx_compress(data, len, &compressed_data, &send_len))
+			{
+				zbx_set_socket_strerror("cannot compress data: %s", zbx_compress_strerror());
+				ret = FAIL;
+				goto cleanup;
+			}
 
-		len64_le = zbx_htole_uint64((zbx_uint64_t)len);
-		memcpy(header_buf + ZBX_TCP_HEADER_LEN, &len64_le, sizeof(len64_le));
+			data = compressed_data;
+			reserved = len;
+		}
 
-		take_bytes = MIN(len, ZBX_TLS_MAX_REC_LEN - ZBX_TCP_HEADER_LEN - sizeof(len64_le));
-		memcpy(header_buf + ZBX_TCP_HEADER_LEN + sizeof(len64_le), data, take_bytes);
+		memcpy(header_buf, ZBX_TCP_HEADER_DATA, ZBX_CONST_STRLEN(ZBX_TCP_HEADER_DATA));
+		offset = ZBX_CONST_STRLEN(ZBX_TCP_HEADER_DATA);
 
-		send_bytes = ZBX_TCP_HEADER_LEN + sizeof(len64_le) + take_bytes;
+		header_buf[offset++] = flags;
+
+		len32_le = zbx_htole_uint32((zbx_uint32_t)send_len);
+		memcpy(header_buf + offset, &len32_le, sizeof(len32_le));
+		offset += sizeof(len32_le);
+
+		len32_le = zbx_htole_uint32((zbx_uint32_t)reserved);
+		memcpy(header_buf + offset, &len32_le, sizeof(len32_le));
+		offset += sizeof(len32_le);
+
+		take_bytes = MIN(send_len, ZBX_TLS_MAX_REC_LEN - offset);
+		memcpy(header_buf + offset, data, take_bytes);
+
+		send_bytes = offset + take_bytes;
 
 		while (written < (ssize_t)send_bytes)
 		{
@@ -775,15 +826,15 @@ int	zbx_tcp_send_ext(zbx_socket_t *s, const char *data, size_t len, unsigned cha
 			written += bytes_sent;
 		}
 
-		written -= ZBX_TCP_HEADER_LEN + (ssize_t)sizeof(len64_le);
+		written -= offset;
 	}
 
-	while (written < (ssize_t)len)
+	while (written < (ssize_t)send_len)
 	{
 		if (ZBX_TCP_SEC_UNENCRYPTED == s->connection_type)
-			send_bytes = len - (size_t)written;
+			send_bytes = send_len - (size_t)written;
 		else
-			send_bytes = MIN(ZBX_TLS_MAX_REC_LEN, len - (size_t)written);
+			send_bytes = MIN(ZBX_TLS_MAX_REC_LEN, send_len - (size_t)written);
 
 		if (ZBX_PROTO_ERROR == (bytes_sent = zbx_tcp_write(s, data + written, send_bytes)))
 		{
@@ -793,6 +844,8 @@ int	zbx_tcp_send_ext(zbx_socket_t *s, const char *data, size_t len, unsigned cha
 		written += bytes_sent;
 	}
 cleanup:
+	zbx_free(compressed_data);
+
 	if (0 != timeout)
 		zbx_socket_timeout_cleanup(s);
 
@@ -889,8 +942,18 @@ int	zbx_tcp_listen(zbx_socket_t *s, const char *listen_ip, unsigned short listen
 	struct addrinfo	hints, *ai = NULL, *current_ai;
 	char		port[8], *ip, *ips, *delim;
 	int		i, err, on, ret = FAIL;
+#ifdef _WINDOWS
+	/* WSASocket() option to prevent inheritance is available on */
+	/* Windows Server 2008 R2 SP1 or newer and on Windows 7 SP1 or newer */
+	static int	no_inherit_wsapi = -1;
 
-	ZBX_SOCKET_START();
+	if (-1 == no_inherit_wsapi)
+	{
+		/* Both Windows 7 and Windows 2008 R2 are 0x0601 */
+		no_inherit_wsapi = zbx_is_win_ver_or_greater((_WIN32_WINNT_WIN7 >> 8) & 0xff,
+				_WIN32_WINNT_WIN7 & 0xff, 1) == SUCCEED;
+	}
+#endif
 
 	zbx_socket_clean(s);
 
@@ -927,15 +990,27 @@ int	zbx_tcp_listen(zbx_socket_t *s, const char *listen_ip, unsigned short listen
 			if (PF_INET != current_ai->ai_family && PF_INET6 != current_ai->ai_family)
 				continue;
 
+#ifdef _WINDOWS
+			/* WSA_FLAG_NO_HANDLE_INHERIT prevents socket inheritance if we call CreateProcess() */
+			/* later on. If it's not available we still try to avoid inheritance by calling  */
+			/* SetHandleInformation() below. WSA_FLAG_OVERLAPPED is not mandatory but strongly */
+			/* recommended for every socket */
+			s->sockets[s->num_socks] = WSASocket(current_ai->ai_family, current_ai->ai_socktype,
+					current_ai->ai_protocol, NULL, 0,
+					(0 != no_inherit_wsapi ? WSA_FLAG_NO_HANDLE_INHERIT : 0) |
+					WSA_FLAG_OVERLAPPED);
+			if (ZBX_SOCKET_ERROR == s->sockets[s->num_socks])
+			{
+				zbx_set_socket_strerror("WSASocket() for [[%s]:%s] failed: %s",
+						ip ? ip : "-", port, strerror_from_system(zbx_socket_last_error()));
+				if (WSAEAFNOSUPPORT == zbx_socket_last_error())
+#else
 			if (ZBX_SOCKET_ERROR == (s->sockets[s->num_socks] =
 					socket(current_ai->ai_family, current_ai->ai_socktype | SOCK_CLOEXEC,
 					current_ai->ai_protocol)))
 			{
 				zbx_set_socket_strerror("socket() for [[%s]:%s] failed: %s",
 						ip ? ip : "-", port, strerror_from_system(zbx_socket_last_error()));
-#ifdef _WINDOWS
-				if (WSAEAFNOSUPPORT == zbx_socket_last_error())
-#else
 				if (EAFNOSUPPORT == zbx_socket_last_error())
 #endif
 					continue;
@@ -948,6 +1023,18 @@ int	zbx_tcp_listen(zbx_socket_t *s, const char *listen_ip, unsigned short listen
 #endif
 			on = 1;
 #ifdef _WINDOWS
+			/* If WSA_FLAG_NO_HANDLE_INHERIT not available, prevent listening socket from */
+			/* inheritance with the old API. Disabling handle inheritance in WSASocket() instead of */
+			/* SetHandleInformation() is preferred because it provides atomicity and gets the job done */
+			/* on systems with non-IFS LSPs installed. So there is a chance that the socket will be still */
+			/* inherited on Windows XP with 3rd party firewall/antivirus installed */
+			if (0 == no_inherit_wsapi && 0 == SetHandleInformation((HANDLE)s->sockets[s->num_socks],
+					HANDLE_FLAG_INHERIT, 0))
+			{
+				zabbix_log(LOG_LEVEL_WARNING, "SetHandleInformation() failed: %s",
+						strerror_from_system(GetLastError()));
+			}
+
 			/* prevent other processes from binding to the same port */
 			/* SO_EXCLUSIVEADDRUSE is mutually exclusive with SO_REUSEADDR */
 			/* on Windows SO_REUSEADDR has different semantics than on Unix */
@@ -982,7 +1069,7 @@ int	zbx_tcp_listen(zbx_socket_t *s, const char *listen_ip, unsigned short listen
 						strerror_from_system(zbx_socket_last_error()));
 			}
 #endif
-			if (ZBX_PROTO_ERROR == bind(s->sockets[s->num_socks], current_ai->ai_addr,
+			if (ZBX_PROTO_ERROR == zbx_bind(s->sockets[s->num_socks], current_ai->ai_addr,
 					current_ai->ai_addrlen))
 			{
 				zbx_set_socket_strerror("bind() for [[%s]:%s] failed: %s",
@@ -1051,8 +1138,18 @@ int	zbx_tcp_listen(zbx_socket_t *s, const char *listen_ip, unsigned short listen
 	ZBX_SOCKADDR	serv_addr;
 	char		*ip, *ips, *delim;
 	int		i, on, ret = FAIL;
+#ifdef _WINDOWS
+	/* WSASocket() option to prevent inheritance is available on */
+	/* Windows Server 2008 R2 or newer and on Windows 7 SP1 or newer */
+	static int	no_inherit_wsapi = -1;
 
-	ZBX_SOCKET_START();
+	if (-1 == no_inherit_wsapi)
+	{
+		/* Both Windows 7 and Windows 2008 R2 are 0x0601 */
+		no_inherit_wsapi = zbx_is_win_ver_or_greater((_WIN32_WINNT_WIN7 >> 8) & 0xff,
+				_WIN32_WINNT_WIN7 & 0xff, 1) == SUCCEED;
+	}
+#endif
 
 	zbx_socket_clean(s);
 
@@ -1077,10 +1174,23 @@ int	zbx_tcp_listen(zbx_socket_t *s, const char *listen_ip, unsigned short listen
 			goto out;
 		}
 
+#ifdef _WINDOWS
+		/* WSA_FLAG_NO_HANDLE_INHERIT prevents socket inheritance if we call CreateProcess() */
+		/* later on. If it's not available we still try to avoid inheritance by calling  */
+		/* SetHandleInformation() below. WSA_FLAG_OVERLAPPED is not mandatory but strongly */
+		/* recommended for every socket */
+		s->sockets[s->num_socks] = WSASocket(AF_INET, SOCK_STREAM, IPPROTO_TCP, NULL, 0,
+				(0 != no_inherit_wsapi ? WSA_FLAG_NO_HANDLE_INHERIT : 0) | WSA_FLAG_OVERLAPPED);
+		if (ZBX_SOCKET_ERROR == s->sockets[s->num_socks])
+		{
+			zbx_set_socket_strerror("WSASocket() for [[%s]:%hu] failed: %s",
+					ip ? ip : "-", listen_port, strerror_from_system(zbx_socket_last_error()));
+#else
 		if (ZBX_SOCKET_ERROR == (s->sockets[s->num_socks] = socket(AF_INET, SOCK_STREAM | SOCK_CLOEXEC, 0)))
 		{
 			zbx_set_socket_strerror("socket() for [[%s]:%hu] failed: %s",
 					ip ? ip : "-", listen_port, strerror_from_system(zbx_socket_last_error()));
+#endif
 			goto out;
 		}
 
@@ -1089,6 +1199,18 @@ int	zbx_tcp_listen(zbx_socket_t *s, const char *listen_ip, unsigned short listen
 #endif
 		on = 1;
 #ifdef _WINDOWS
+		/* If WSA_FLAG_NO_HANDLE_INHERIT not available, prevent listening socket from */
+		/* inheritance with the old API. Disabling handle inheritance in WSASocket() instead of */
+		/* SetHandleInformation() is preferred because it provides atomicity and gets the job done */
+		/* on systems with non-IFS LSPs installed. So there is a chance that the socket will be still */
+		/* inherited on Windows XP with 3rd party firewall/antivirus installed */
+		if (0 == no_inherit_wsapi && 0 == SetHandleInformation((HANDLE)s->sockets[s->num_socks],
+				HANDLE_FLAG_INHERIT, 0))
+		{
+			zabbix_log(LOG_LEVEL_WARNING, "SetHandleInformation() failed: %s",
+					strerror_from_system(GetLastError()));
+		}
+
 		/* prevent other processes from binding to the same port */
 		/* SO_EXCLUSIVEADDRUSE is mutually exclusive with SO_REUSEADDR */
 		/* on Windows SO_REUSEADDR has different semantics than on Unix */
@@ -1446,7 +1568,8 @@ const char	*zbx_tcp_recv_line(zbx_socket_t *s)
 		}
 		else
 		{
-			if (0 != (left = MIN(ZBX_TCP_LINE_LEN - s->read_bytes, (size_t)(ptr - buffer))))
+			if (0 != (left = (NULL == ptr ? ZBX_TCP_LINE_LEN - s->read_bytes :
+					MIN(ZBX_TCP_LINE_LEN - s->read_bytes, (size_t)(ptr - buffer)))))
 			{
 				/* fill the string to the defined limit */
 				zbx_strncpy_alloc(&s->buffer, &alloc, &offset, buffer, left);
@@ -1530,19 +1653,21 @@ static ssize_t	zbx_tcp_read(zbx_socket_t *s, char *buf, size_t len)
  * Author: Eugene Grigorjev                                                   *
  *                                                                            *
  ******************************************************************************/
-ssize_t	zbx_tcp_recv_ext(zbx_socket_t *s, unsigned char flags, int timeout)
+ssize_t	zbx_tcp_recv_ext(zbx_socket_t *s, int timeout)
 {
-#define ZBX_TCP_EXPECT_HEADER	1
-#define ZBX_TCP_EXPECT_LENGTH	2
-#define ZBX_TCP_EXPECT_TEXT_XML	3
-#define ZBX_TCP_EXPECT_SIZE	4
-#define ZBX_TCP_EXPECT_CLOSE	5
-#define ZBX_TCP_EXPECT_XML_END	6
+#define ZBX_TCP_EXPECT_HEADER		1
+#define ZBX_TCP_EXPECT_VERSION		2
+#define ZBX_TCP_EXPECT_VERSION_VALIDATE	3
+#define ZBX_TCP_EXPECT_LENGTH		4
+#define ZBX_TCP_EXPECT_SIZE		5
+
+	const char	*__function_name = "zbx_tcp_recv_ext";
 
 	ssize_t		nbytes;
-	size_t		allocated = 8 * ZBX_STAT_BUF_LEN, buf_dyn_bytes = 0, buf_stat_bytes = 0, header_bytes = 0;
-	zbx_uint64_t	expected_len = 16 * ZBX_MEBIBYTE;
+	size_t		buf_dyn_bytes = 0, buf_stat_bytes = 0, offset = 0;
+	zbx_uint32_t	expected_len = 16 * ZBX_MEBIBYTE, reserved = 0;
 	unsigned char	expect = ZBX_TCP_EXPECT_HEADER;
+	int		protocol_version;
 
 	if (0 != timeout)
 		zbx_socket_timeout_set(s, timeout);
@@ -1560,144 +1685,266 @@ ssize_t	zbx_tcp_recv_ext(zbx_socket_t *s, unsigned char flags, int timeout)
 		if (ZBX_BUF_TYPE_STAT == s->buf_type)
 			buf_stat_bytes += nbytes;
 		else
-			zbx_strncpy_alloc(&s->buffer, &allocated, &buf_dyn_bytes, s->buf_stat, nbytes);
+		{
+			if (buf_dyn_bytes + nbytes <= expected_len)
+				memcpy(s->buffer + buf_dyn_bytes, s->buf_stat, nbytes);
+			buf_dyn_bytes += nbytes;
+		}
 
 		if (buf_stat_bytes + buf_dyn_bytes >= expected_len)
 			break;
 
-		/* performance short-circuit, can be omitted */
-		if (ZBX_TCP_EXPECT_SIZE == expect || (ZBX_TCP_EXPECT_CLOSE == expect && ZBX_BUF_TYPE_DYN == s->buf_type))
-			continue;
-
 		if (ZBX_TCP_EXPECT_HEADER == expect)
 		{
 			if (ZBX_TCP_HEADER_LEN > buf_stat_bytes)
 			{
-				if (0 == strncmp(s->buf_stat, ZBX_TCP_HEADER, buf_stat_bytes))
+				if (0 == strncmp(s->buf_stat, ZBX_TCP_HEADER_DATA, buf_stat_bytes))
 					continue;
 
-				expect = ZBX_TCP_EXPECT_TEXT_XML;
+				break;
 			}
 			else
 			{
-				if (0 == strncmp(s->buf_stat, ZBX_TCP_HEADER, ZBX_TCP_HEADER_LEN))
-					expect = ZBX_TCP_EXPECT_LENGTH;
-				else
-					expect = ZBX_TCP_EXPECT_TEXT_XML;
+				if (0 != strncmp(s->buf_stat, ZBX_TCP_HEADER_DATA, ZBX_TCP_HEADER_LEN))
+				{
+					/* invalid header, abort receiving */
+					break;
+				}
+
+				expect = ZBX_TCP_EXPECT_VERSION;
+				offset += ZBX_TCP_HEADER_LEN;
 			}
 		}
 
+		if (ZBX_TCP_EXPECT_VERSION == expect)
+		{
+			if (offset + 1 > buf_stat_bytes)
+				continue;
+
+			expect = ZBX_TCP_EXPECT_VERSION_VALIDATE;
+			protocol_version = s->buf_stat[ZBX_TCP_HEADER_LEN];
+
+			if (0 == (protocol_version & ZBX_TCP_PROTOCOL) ||
+					protocol_version > (ZBX_TCP_PROTOCOL | ZBX_TCP_COMPRESS))
+			{
+				/* invalid protocol version, abort receiving */
+				break;
+			}
+			s->protocol = protocol_version;
+			expect = ZBX_TCP_EXPECT_LENGTH;
+			offset++;
+		}
+
 		if (ZBX_TCP_EXPECT_LENGTH == expect)
 		{
-			if (ZBX_TCP_HEADER_LEN + sizeof(zbx_uint64_t) > buf_stat_bytes)
+			if (offset + 2 * sizeof(zbx_uint32_t) > buf_stat_bytes)
 				continue;
 
-			memcpy(&expected_len, s->buf_stat + ZBX_TCP_HEADER_LEN, sizeof(zbx_uint64_t));
-			expected_len = zbx_letoh_uint64(expected_len);
+			memcpy(&expected_len, s->buf_stat + offset, sizeof(zbx_uint32_t));
+			offset += sizeof(zbx_uint32_t);
+			expected_len = zbx_letoh_uint32(expected_len);
+
+			memcpy(&reserved, s->buf_stat + offset, sizeof(zbx_uint32_t));
+			offset += sizeof(zbx_uint32_t);
+			reserved = zbx_letoh_uint32(reserved);
 
 			if (ZBX_MAX_RECV_DATA_SIZE < expected_len)
 			{
 				zabbix_log(LOG_LEVEL_WARNING, "Message size " ZBX_FS_UI64 " from %s exceeds the "
-						"maximum size " ZBX_FS_UI64 " bytes. Message ignored.", expected_len,
-						s->peer, (zbx_uint64_t)ZBX_MAX_RECV_DATA_SIZE);
+						"maximum size " ZBX_FS_UI64 " bytes. Message ignored.",
+						(zbx_uint64_t)expected_len, s->peer,
+						(zbx_uint64_t)ZBX_MAX_RECV_DATA_SIZE);
+				nbytes = ZBX_PROTO_ERROR;
+				goto out;
+			}
+
+			/* compressed protocol stores uncompressed packet size in the reserved data */
+			if (0 != (protocol_version & ZBX_TCP_COMPRESS) && ZBX_MAX_RECV_DATA_SIZE < reserved)
+			{
+				zabbix_log(LOG_LEVEL_WARNING, "Uncompressed message size " ZBX_FS_UI64
+						" from %s exceeds the maximum size " ZBX_FS_UI64
+						" bytes. Message ignored.", (zbx_uint64_t)reserved, s->peer,
+						(zbx_uint64_t)ZBX_MAX_RECV_DATA_SIZE);
 				nbytes = ZBX_PROTO_ERROR;
 				goto out;
 			}
 
 			if (sizeof(s->buf_stat) > expected_len)
 			{
-				buf_stat_bytes -= ZBX_TCP_HEADER_LEN + sizeof(zbx_uint64_t);
-				memmove(s->buf_stat, s->buf_stat + ZBX_TCP_HEADER_LEN + sizeof(zbx_uint64_t),
-						buf_stat_bytes);
+				buf_stat_bytes -= offset;
+				memmove(s->buf_stat, s->buf_stat + offset, buf_stat_bytes);
 			}
 			else
 			{
 				s->buf_type = ZBX_BUF_TYPE_DYN;
-				s->buffer = zbx_malloc(NULL, allocated);
-				buf_dyn_bytes = buf_stat_bytes - ZBX_TCP_HEADER_LEN - sizeof(zbx_uint64_t);
+				s->buffer = (char *)zbx_malloc(NULL, expected_len + 1);
+				buf_dyn_bytes = buf_stat_bytes - offset;
 				buf_stat_bytes = 0;
-				memcpy(s->buffer, s->buf_stat + ZBX_TCP_HEADER_LEN + sizeof(zbx_uint64_t),
-						buf_dyn_bytes);
+				memcpy(s->buffer, s->buf_stat + offset, buf_dyn_bytes);
 			}
 
 			expect = ZBX_TCP_EXPECT_SIZE;
-			header_bytes = ZBX_TCP_HEADER_LEN + sizeof(zbx_uint64_t);
 
 			if (buf_stat_bytes + buf_dyn_bytes >= expected_len)
 				break;
-
-			continue;
-		}
-
-		if (sizeof(s->buf_stat) == buf_stat_bytes)
-		{
-			s->buf_type = ZBX_BUF_TYPE_DYN;
-			s->buffer = zbx_malloc(NULL, allocated);
-			buf_dyn_bytes = sizeof(s->buf_stat);
-			buf_stat_bytes = 0;
-			memcpy(s->buffer, s->buf_stat, sizeof(s->buf_stat));
-			continue;
-		}
-
-		if (sizeof(s->buf_stat) == nbytes)
-			continue;
-
-		if (ZBX_TCP_EXPECT_TEXT_XML == expect)
-		{
-			if (0 != (flags & ZBX_TCP_READ_UNTIL_CLOSE))
-			{
-				expect = ZBX_TCP_EXPECT_CLOSE;
-				continue;
-			}
-
-			if (ZBX_CONST_STRLEN("<req>") > buf_stat_bytes + buf_dyn_bytes)
-			{
-				if (0 != strncmp(s->buffer, "<req>", buf_stat_bytes + buf_dyn_bytes))
-					break;
-
-				continue;
-			}
-			else
-			{
-				if (0 != strncmp(s->buffer, "<req>", ZBX_CONST_STRLEN("<req>")))
-					break;
-
-				expect = ZBX_TCP_EXPECT_XML_END;
-			}
-		}
-
-		if (ZBX_TCP_EXPECT_XML_END == expect)
-		{
-			/* closing tag received in the last 10 bytes? */
-			s->buffer[buf_stat_bytes + buf_dyn_bytes] = '\0';
-			if (NULL != strstr(s->buffer + buf_stat_bytes + buf_dyn_bytes - (10 > buf_stat_bytes +
-					buf_dyn_bytes ? buf_stat_bytes + buf_dyn_bytes : 10), "</req>"))
-			{
-				break;
-			}
 		}
 	}
 
 	if (ZBX_TCP_EXPECT_SIZE == expect)
 	{
-		if (buf_stat_bytes + buf_dyn_bytes != expected_len)
+		if (buf_stat_bytes + buf_dyn_bytes == expected_len)
+		{
+			if (0 != (protocol_version & ZBX_TCP_COMPRESS))
+			{
+				char	*out;
+				size_t	out_size = reserved;
+
+				out = (char *)zbx_malloc(NULL, reserved + 1);
+				if (FAIL == zbx_uncompress(s->buffer, buf_stat_bytes + buf_dyn_bytes, out, &out_size))
+				{
+					zbx_free(out);
+					zbx_set_socket_strerror("cannot uncompress data: %s", zbx_compress_strerror());
+					nbytes = ZBX_PROTO_ERROR;
+					goto out;
+				}
+
+				if (out_size != reserved)
+				{
+					zbx_free(out);
+					zbx_set_socket_strerror("size of uncompressed data is less than expected");
+					nbytes = ZBX_PROTO_ERROR;
+					goto out;
+				}
+
+				if (ZBX_BUF_TYPE_DYN == s->buf_type)
+					zbx_free(s->buffer);
+
+				s->buf_type = ZBX_BUF_TYPE_DYN;
+				s->buffer = out;
+				s->read_bytes = reserved;
+
+				zabbix_log(LOG_LEVEL_TRACE, "%s(): received " ZBX_FS_SIZE_T " bytes with"
+						" compression ratio %.1f", __function_name,
+						(zbx_fs_size_t)(buf_stat_bytes + buf_dyn_bytes),
+						(double)reserved / (buf_stat_bytes + buf_dyn_bytes));
+			}
+			else
+				s->read_bytes = buf_stat_bytes + buf_dyn_bytes;
+
+			s->buffer[s->read_bytes] = '\0';
+		}
+		else
 		{
 			if (buf_stat_bytes + buf_dyn_bytes < expected_len)
 			{
 				zabbix_log(LOG_LEVEL_WARNING, "Message from %s is shorter than expected " ZBX_FS_UI64
-						" bytes. Message ignored.", s->peer, expected_len);
+						" bytes. Message ignored.", s->peer, (zbx_uint64_t)expected_len);
 			}
 			else
 			{
 				zabbix_log(LOG_LEVEL_WARNING, "Message from %s is longer than expected " ZBX_FS_UI64
-						" bytes. Message ignored.", s->peer, expected_len);
+						" bytes. Message ignored.", s->peer, (zbx_uint64_t)expected_len);
 			}
 
 			nbytes = ZBX_PROTO_ERROR;
-			goto out;
 		}
 	}
-	else if (buf_stat_bytes + buf_dyn_bytes >= expected_len)
+	else if (ZBX_TCP_EXPECT_LENGTH == expect)
+	{
+		zabbix_log(LOG_LEVEL_WARNING, "Message from %s is missing data length. Message ignored.", s->peer);
+		nbytes = ZBX_PROTO_ERROR;
+	}
+	else if (ZBX_TCP_EXPECT_VERSION == expect)
+	{
+		zabbix_log(LOG_LEVEL_WARNING, "Message from %s is missing protocol version. Message ignored.",
+				s->peer);
+		nbytes = ZBX_PROTO_ERROR;
+	}
+	else if (ZBX_TCP_EXPECT_VERSION_VALIDATE == expect)
+	{
+		zabbix_log(LOG_LEVEL_WARNING, "Message from %s is using unsupported protocol version \"%d\"."
+				" Message ignored.", s->peer, protocol_version);
+		nbytes = ZBX_PROTO_ERROR;
+	}
+	else if (0 != buf_stat_bytes)
+	{
+		zabbix_log(LOG_LEVEL_WARNING, "Message from %s is missing header. Message ignored.", s->peer);
+		nbytes = ZBX_PROTO_ERROR;
+	}
+	else
+	{
+		s->read_bytes = 0;
+		s->buffer[s->read_bytes] = '\0';
+	}
+out:
+	if (0 != timeout)
+		zbx_socket_timeout_cleanup(s);
+
+	return (ZBX_PROTO_ERROR == nbytes ? FAIL : (ssize_t)(s->read_bytes + offset));
+
+#undef ZBX_TCP_EXPECT_HEADER
+#undef ZBX_TCP_EXPECT_LENGTH
+#undef ZBX_TCP_EXPECT_SIZE
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Function: zbx_tcp_recv_raw_ext                                             *
+ *                                                                            *
+ * Purpose: receive data till connection is closed                            *
+ *                                                                            *
+ * Return value: number of bytes received - success,                          *
+ *               FAIL - an error occurred                                     *
+ *                                                                            *
+ ******************************************************************************/
+ssize_t	zbx_tcp_recv_raw_ext(zbx_socket_t *s, int timeout)
+{
+	ssize_t		nbytes;
+	size_t		allocated = 8 * ZBX_STAT_BUF_LEN, buf_dyn_bytes = 0, buf_stat_bytes = 0;
+	zbx_uint64_t	expected_len = 16 * ZBX_MEBIBYTE;
+
+	if (0 != timeout)
+		zbx_socket_timeout_set(s, timeout);
+
+	zbx_socket_free(s);
+
+	s->buf_type = ZBX_BUF_TYPE_STAT;
+	s->buffer = s->buf_stat;
+
+	while (0 != (nbytes = zbx_tcp_read(s, s->buf_stat + buf_stat_bytes, sizeof(s->buf_stat) - buf_stat_bytes)))
+	{
+		if (ZBX_PROTO_ERROR == nbytes)
+			goto out;
+
+		if (ZBX_BUF_TYPE_STAT == s->buf_type)
+			buf_stat_bytes += nbytes;
+		else
+		{
+			if (buf_dyn_bytes + nbytes >= allocated)
+			{
+				while (buf_dyn_bytes + nbytes >= allocated)
+					allocated *= 2;
+				s->buffer = (char *)zbx_realloc(s->buffer, allocated);
+			}
+
+			memcpy(s->buffer + buf_dyn_bytes, s->buf_stat, nbytes);
+			buf_dyn_bytes += nbytes;
+		}
+
+		if (buf_stat_bytes + buf_dyn_bytes >= expected_len)
+			break;
+
+		if (sizeof(s->buf_stat) == buf_stat_bytes)
+		{
+			s->buf_type = ZBX_BUF_TYPE_DYN;
+			s->buffer = (char *)zbx_malloc(NULL, allocated);
+			buf_dyn_bytes = sizeof(s->buf_stat);
+			buf_stat_bytes = 0;
+			memcpy(s->buffer, s->buf_stat, sizeof(s->buf_stat));
+		}
+	}
+
+	if (buf_stat_bytes + buf_dyn_bytes >= expected_len)
 	{
 		zabbix_log(LOG_LEVEL_WARNING, "Message from %s is longer than " ZBX_FS_UI64 " bytes allowed for"
 				" plain text. Message ignored.", s->peer, expected_len);
@@ -1711,17 +1958,10 @@ out:
 	if (0 != timeout)
 		zbx_socket_timeout_cleanup(s);
 
-	return (ZBX_PROTO_ERROR == nbytes ? FAIL : (ssize_t)(s->read_bytes + header_bytes));
-
-#undef ZBX_TCP_EXPECT_HEADER
-#undef ZBX_TCP_EXPECT_LENGTH
-#undef ZBX_TCP_EXPECT_TEXT_XML
-#undef ZBX_TCP_EXPECT_SIZE
-#undef ZBX_TCP_EXPECT_CLOSE
-#undef ZBX_TCP_EXPECT_XML_END
+	return (ZBX_PROTO_ERROR == nbytes ? FAIL : (ssize_t)(s->read_bytes));
 }
 
-static int	subnet_match(int af, unsigned int prefix_size, void *address1, void *address2)
+static int	subnet_match(int af, unsigned int prefix_size, const void *address1, const void *address2)
 {
 	unsigned char	netmask[16] = {0};
 	int		i, j, bytes;
@@ -1747,8 +1987,11 @@ static int	subnet_match(int af, unsigned int prefix_size, void *address1, void *
 	/* All hosts on a subnetwork have the same network prefix. */
 	for (i = 0; i < bytes; i++)
 	{
-		if ((((unsigned char *)address1)[i] & netmask[i]) != (((unsigned char *)address2)[i] & netmask[i]))
+		if ((((const unsigned char *)address1)[i] & netmask[i]) !=
+				(((const unsigned char *)address2)[i] & netmask[i]))
+		{
 			return FAIL;
+		}
 	}
 
 	return SUCCEED;
@@ -1900,9 +2143,10 @@ int	zbx_validate_peer_list(const char *peer_list, char **error)
  *           the same: 127.0.0.1 == ::127.0.0.1 == ::ffff:127.0.0.1           *
  *                                                                            *
  ******************************************************************************/
-int	zbx_tcp_check_allowed_peers(zbx_socket_t *s, const char *peer_list)
+int	zbx_tcp_check_allowed_peers(const zbx_socket_t *s, const char *peer_list)
 {
 	char	*start = NULL, *end = NULL, *cidr_sep, tmp[MAX_STRING_LEN];
+	int	prefix_size;
 
 	/* examine list of allowed peers which may include DNS names, IPv4/6 addresses and addresses in CIDR notation */
 
@@ -1911,18 +2155,13 @@ int	zbx_tcp_check_allowed_peers(zbx_socket_t *s, const char *peer_list)
 	for (start = tmp; '\0' != *start;)
 	{
 #ifdef HAVE_IPV6
-#ifdef HAVE_SOCKADDR_STORAGE_SS_FAMILY
-		int	ai_family = s->peer_info.ss_family;
-#else
-		int	ai_family = s->peer_info.__ss_family;
-#endif
-		unsigned int	prefix_size = (ai_family == AF_INET ? IPV4_MAX_CIDR_PREFIX : IPV6_MAX_CIDR_PREFIX);
 		struct addrinfo	hints, *ai = NULL, *current_ai;
 #else
-		unsigned int	prefix_size = IPV4_MAX_CIDR_PREFIX;
 		struct hostent	*hp;
 #endif /* HAVE_IPV6 */
 
+		prefix_size = -1;
+
 		if (NULL != (end = strchr(start, ',')))
 			*end = '\0';
 
@@ -1941,7 +2180,7 @@ int	zbx_tcp_check_allowed_peers(zbx_socket_t *s, const char *peer_list)
 
 #ifdef HAVE_IPV6
 		memset(&hints, 0, sizeof(hints));
-		hints.ai_family = ai_family;
+		hints.ai_family = AF_UNSPEC;
 		hints.ai_socktype = SOCK_STREAM;
 		hints.ai_protocol = IPPROTO_TCP;
 
@@ -1949,7 +2188,15 @@ int	zbx_tcp_check_allowed_peers(zbx_socket_t *s, const char *peer_list)
 		{
 			for (current_ai = ai; NULL != current_ai; current_ai = current_ai->ai_next)
 			{
-				if (SUCCEED == zbx_ip_cmp(prefix_size, current_ai, s->peer_info))
+				int	prefix_size_current = prefix_size;
+
+				if (-1 == prefix_size_current)
+				{
+					prefix_size_current = (current_ai->ai_family == AF_INET ?
+							IPV4_MAX_CIDR_PREFIX : IPV6_MAX_CIDR_PREFIX);
+				}
+
+				if (SUCCEED == zbx_ip_cmp(prefix_size_current, current_ai, s->peer_info))
 				{
 					freeaddrinfo(ai);
 					return SUCCEED;
@@ -1964,6 +2211,9 @@ int	zbx_tcp_check_allowed_peers(zbx_socket_t *s, const char *peer_list)
 
 			for (i = 0; NULL != hp->h_addr_list[i]; i++)
 			{
+				if (-1 == prefix_size)
+					prefix_size = IPV4_MAX_CIDR_PREFIX;
+
 				if (SUCCEED == subnet_match(AF_INET, prefix_size,
 						&((struct in_addr *)hp->h_addr_list[i])->s_addr,
 						&s->peer_info.sin_addr.s_addr))
@@ -2018,7 +2268,7 @@ int	zbx_udp_send(zbx_socket_t *s, const char *data, size_t data_len, int timeout
 	if (0 != timeout)
 		zbx_socket_timeout_set(s, timeout);
 
-	if (ZBX_PROTO_ERROR == sendto(s->socket, data, data_len, 0, NULL, 0))
+	if (ZBX_PROTO_ERROR == zbx_sendto(s->socket, data, data_len, 0, NULL, 0))
 	{
 		zbx_set_socket_strerror("sendto() failed: %s", strerror_from_system(zbx_socket_last_error()));
 		ret = FAIL;
@@ -2057,7 +2307,7 @@ int	zbx_udp_recv(zbx_socket_t *s, int timeout)
 	else
 	{
 		s->buf_type = ZBX_BUF_TYPE_DYN;
-		s->buffer = zbx_malloc(s->buffer, read_bytes + 1);
+		s->buffer = (char *)zbx_malloc(s->buffer, read_bytes + 1);
 	}
 
 	buffer[read_bytes] = '\0';
