@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -144,17 +144,17 @@ static int	history_parse_value(struct zbx_json_parse *jp, unsigned char value_ty
 	size_t	value_alloc = 0;
 	int	ret = FAIL;
 
-	if (SUCCEED != zbx_json_value_by_name_dyn(jp, "clock", &value, &value_alloc))
+	if (SUCCEED != zbx_json_value_by_name_dyn(jp, "clock", &value, &value_alloc, NULL))
 		goto out;
 
 	hr->timestamp.sec = atoi(value);
 
-	if (SUCCEED != zbx_json_value_by_name_dyn(jp, "ns", &value, &value_alloc))
+	if (SUCCEED != zbx_json_value_by_name_dyn(jp, "ns", &value, &value_alloc, NULL))
 		goto out;
 
 	hr->timestamp.ns = atoi(value);
 
-	if (SUCCEED != zbx_json_value_by_name_dyn(jp, "value", &value, &value_alloc))
+	if (SUCCEED != zbx_json_value_by_name_dyn(jp, "value", &value, &value_alloc, NULL))
 		goto out;
 
 	hr->value = history_str2value(value, value_type);
@@ -162,22 +162,22 @@ static int	history_parse_value(struct zbx_json_parse *jp, unsigned char value_ty
 	if (ITEM_VALUE_TYPE_LOG == value_type)
 	{
 
-		if (SUCCEED != zbx_json_value_by_name_dyn(jp, "timestamp", &value, &value_alloc))
+		if (SUCCEED != zbx_json_value_by_name_dyn(jp, "timestamp", &value, &value_alloc, NULL))
 			goto out;
 
 		hr->value.log->timestamp = atoi(value);
 
-		if (SUCCEED != zbx_json_value_by_name_dyn(jp, "logeventid", &value, &value_alloc))
+		if (SUCCEED != zbx_json_value_by_name_dyn(jp, "logeventid", &value, &value_alloc, NULL))
 			goto out;
 
 		hr->value.log->logeventid = atoi(value);
 
-		if (SUCCEED != zbx_json_value_by_name_dyn(jp, "severity", &value, &value_alloc))
+		if (SUCCEED != zbx_json_value_by_name_dyn(jp, "severity", &value, &value_alloc, NULL))
 			goto out;
 
 		hr->value.log->severity = atoi(value);
 
-		if (SUCCEED != zbx_json_value_by_name_dyn(jp, "source", &value, &value_alloc))
+		if (SUCCEED != zbx_json_value_by_name_dyn(jp, "source", &value, &value_alloc, NULL))
 			goto out;
 
 		hr->value.log->source = zbx_strdup(NULL, value);
@@ -193,19 +193,24 @@ out:
 
 static void	elastic_log_error(CURL *handle, CURLcode error, const char *errbuf)
 {
-	long	http_code;
+	char		http_status[MAX_STRING_LEN];
+	long int	http_code;
+	CURLcode	curl_err;
 
 	if (CURLE_HTTP_RETURNED_ERROR == error)
 	{
-		curl_easy_getinfo(handle, CURLINFO_RESPONSE_CODE, &http_code);
+		if (CURLE_OK == (curl_err = curl_easy_getinfo(handle, CURLINFO_RESPONSE_CODE, &http_code)))
+			zbx_snprintf(http_status, sizeof(http_status), "HTTP status code: %ld", http_code);
+		else
+			zbx_strlcpy(http_status, "unknown HTTP status code", sizeof(http_status));
 
 		if (0 != page_r.offset)
 		{
-			zabbix_log(LOG_LEVEL_ERR, "cannot get values from elasticsearch, HTTP error: %ld, message: %s",
-					http_code, page_r.data);
+			zabbix_log(LOG_LEVEL_ERR, "cannot get values from elasticsearch, %s, message: %s", http_status,
+					page_r.data);
 		}
 		else
-			zabbix_log(LOG_LEVEL_ERR, "cannot get values from elasticsearch, HTTP error: %ld", http_code);
+			zabbix_log(LOG_LEVEL_ERR, "cannot get values from elasticsearch, %s", http_status);
 	}
 	else
 	{
@@ -281,17 +286,17 @@ static int	elastic_is_error_present(zbx_httppage_t *page, char **err)
 					SUCCEED == zbx_json_brackets_by_name(&jp_item, "index", &jp_index) &&
 					SUCCEED == zbx_json_brackets_by_name(&jp_index, "error", &jp_error))
 			{
-				if (SUCCEED != zbx_json_value_by_name_dyn(&jp_error, "type", &type, &type_alloc))
+				if (SUCCEED != zbx_json_value_by_name_dyn(&jp_error, "type", &type, &type_alloc, NULL))
 					rc_js = FAIL;
-				if (SUCCEED != zbx_json_value_by_name_dyn(&jp_error, "reason", &reason, &reason_alloc))
+				if (SUCCEED != zbx_json_value_by_name_dyn(&jp_error, "reason", &reason, &reason_alloc, NULL))
 					rc_js = FAIL;
 			}
 			else
 				continue;
 
-			if (SUCCEED != zbx_json_value_by_name_dyn(&jp_index, "status", &status, &status_alloc))
+			if (SUCCEED != zbx_json_value_by_name_dyn(&jp_index, "status", &status, &status_alloc, NULL))
 				rc_js = FAIL;
-			if (SUCCEED != zbx_json_value_by_name_dyn(&jp_index, "_index", &index, &index_alloc))
+			if (SUCCEED != zbx_json_value_by_name_dyn(&jp_index, "_index", &index, &index_alloc, NULL))
 				rc_js = FAIL;
 
 			break;
@@ -480,11 +485,23 @@ try_again:
 				}
 				else
 				{
+					char		http_status[MAX_STRING_LEN];
 					long int	err;
+					CURLcode	curl_err;
 
-					curl_easy_getinfo(msg->easy_handle, CURLINFO_RESPONSE_CODE, &err);
-					zabbix_log(LOG_LEVEL_ERR, "cannot send data to elasticsearch, HTTP error code:"
-							" %ld", err);
+					if (CURLE_OK == (curl_err = curl_easy_getinfo(msg->easy_handle,
+							CURLINFO_RESPONSE_CODE, &err)))
+					{
+						zbx_snprintf(http_status, sizeof(http_status), "HTTP status code: %ld",
+								err);
+					}
+					else
+					{
+						zbx_strlcpy(http_status, "unknown HTTP status code",
+								sizeof(http_status));
+					}
+
+					zabbix_log(LOG_LEVEL_ERR, "cannot send data to elasticsearch, %s", http_status);
 				}
 			}
 			else if (CURLE_OK != msg->data.result)
@@ -710,7 +727,7 @@ static int	elastic_get_values(zbx_history_iface_t *hist, zbx_uint64_t itemid, in
 		zbx_json_brackets_open(jp.start, &jp_values);
 
 		/* get the scroll id immediately, for being used in subsequent queries */
-		if (SUCCEED != zbx_json_value_by_name_dyn(&jp_values, "_scroll_id", &scroll_id, &id_alloc))
+		if (SUCCEED != zbx_json_value_by_name_dyn(&jp_values, "_scroll_id", &scroll_id, &id_alloc, NULL))
 		{
 			zabbix_log(LOG_LEVEL_WARNING, "elasticsearch version is not compatible with zabbix server. "
 					"_scroll_id tag is absent");
