@@ -41,10 +41,11 @@ static unsigned char	source_ip6_checked = 0;
 static const char	*source_ip6_option = NULL;
 #endif
 
-#define FPING_UNINITIALIZED_INTERVAL	-1
-static int		packet_interval = FPING_UNINITIALIZED_INTERVAL;
+#define FPING_UNINITIALIZED_VALUE	-2
+static int		packet_interval = FPING_UNINITIALIZED_VALUE;
 #ifdef HAVE_IPV6
-static int		packet_interval6 = FPING_UNINITIALIZED_INTERVAL;
+static int		packet_interval6 = FPING_UNINITIALIZED_VALUE;
+static int		fping_ipv6_supported = FPING_UNINITIALIZED_VALUE;
 #endif
 
 static void	get_source_ip_option(const char *fping, const char **option, unsigned char *checked)
@@ -86,36 +87,108 @@ static void	get_source_ip_option(const char *fping, const char **option, unsigne
  *                                                                            *
  * Purpose: detect minimal possible fping packet interval                     *
  *                                                                            *
- * Parameters: fping - [IN] the the location of fping program                 *
- *             dst   - [IN] the the ip address for test                       *
+ * Parameters: fping         - [IN] the the location of fping program         *
+ *             dst           - [IN] the the ip address for test               *
+ *             value         - [OUT] interval between sending ping packets    *
+ *                                   (in millisec)                            *
+ *             error         - [OUT] error string if function fails           *
+ *             max_error_len - [IN] length of error buffer                    *
  *                                                                            *
- * Return value: interval between sending ping packets (in millisec)          *
+ * Return value: SUCCEED if processed successfully or FAIL otherwise          *
  *                                                                            *
- * Comments: starting with fping (4.x), the packets interval can be 0ms,      *
+ * Comments: starting with fping (4.x), the packets interval can be 0ms, 1ms, *
  *           otherwise minimum value is 10ms                                  *
  ******************************************************************************/
-static int	get_interval_option(const char * fping, const char *dst)
+static int	get_interval_option(const char * fping, const char *dst, int *value, char *error, int max_error_len)
 {
-	int	value, ret;
-	char	tmp[MAX_STRING_LEN], error[255], *out = NULL;
+	int	ret_exec, ret = FAIL;
+	char	tmp[MAX_STRING_LEN], err[255], *out = NULL;
 
 	zbx_snprintf(tmp, sizeof(tmp), "%s -c1 -t50 -i0 %s", fping, dst);
 
-	if ((SUCCEED == (ret = zbx_execute(tmp, &out, error, sizeof(error), 1, ZBX_EXIT_CODE_CHECKS_DISABLED)) &&
-			ZBX_KIBIBYTE > strlen(out) && NULL != strstr(out, dst)) || TIMEOUT_ERROR == ret)
+	if (SUCCEED == (ret_exec = zbx_execute(tmp, &out, err, sizeof(err), 1, ZBX_EXIT_CODE_CHECKS_DISABLED)) &&
+			ZBX_KIBIBYTE > strlen(out) && NULL != strstr(out, dst))
 	{
-		value = 0;
+		*value = 0;
+		ret = SUCCEED;
+	}
+	else if (TIMEOUT_ERROR == ret_exec)
+	{
+		zbx_snprintf(error, max_error_len, "Timeout while executing: %s", fping);
+	}
+	else if (FAIL == ret_exec)
+		zbx_snprintf(error, max_error_len, "Failed to execute command \"%s\": %s", fping, err);
+
+	zbx_free(out);
+
+	if (SUCCEED == ret || SUCCEED != ret_exec)
+		return ret;
+
+	zbx_snprintf(tmp, sizeof(tmp), "%s -c1 -t50 -i1 %s", fping, dst);
+
+	if (SUCCEED == (ret_exec = zbx_execute(tmp, &out, err, sizeof(err), 1, ZBX_EXIT_CODE_CHECKS_DISABLED))
+			&& ZBX_KIBIBYTE > strlen(out) && NULL != strstr(out, dst))
+	{
+		*value = 1;
+		ret = SUCCEED;
+	}
+	else if (TIMEOUT_ERROR == ret_exec)
+	{
+		zbx_snprintf(error, max_error_len, "Timeout while executing: %s", fping);
+	}
+	else if (FAIL == ret_exec)
+	{
+		zbx_snprintf(error, max_error_len, "Failed to execute command \"%s\": %s", fping, err);
 	}
 	else
 	{
-		value = 10;
+		*value = 10;
+		ret = SUCCEED;
 	}
 
 	zbx_free(out);
 
-	return value;
+	return ret;
 }
 
+#ifdef HAVE_IPV6
+/******************************************************************************
+ *                                                                            *
+ * Function: get_ipv6_support                                                 *
+ *                                                                            *
+ * Purpose: check fping supports IPv6                                         *
+ *                                                                            *
+ * Parameters: fping - [IN] the the location of fping program                 *
+ *             dst   - [IN] the the ip address for test                       *
+ *                                                                            *
+ * Return value: SUCCEED - IPv6 is supported                                  *
+ *               FAIL    - IPv6 is not supported                              *
+ *                                                                            *
+ ******************************************************************************/
+static int	get_ipv6_support(const char * fping, const char *dst)
+{
+	int	ret;
+	char	tmp[MAX_STRING_LEN], error[255], *out = NULL;
+
+	zbx_snprintf(tmp, sizeof(tmp), "%s -6 -c1 -t50 %s", fping, dst);
+
+	if ((SUCCEED == (ret = zbx_execute(tmp, &out, error, sizeof(error), 1, ZBX_EXIT_CODE_CHECKS_DISABLED)) &&
+				ZBX_KIBIBYTE > strlen(out) && NULL != strstr(out, dst)) || TIMEOUT_ERROR == ret)
+	{
+		ret = SUCCEED;
+	}
+	else
+	{
+		ret = FAIL;
+	}
+
+	zbx_free(out);
+
+	return ret;
+
+}
+#endif	/* HAVE_IPV6 */
+
 static int	process_ping(ZBX_FPING_HOST *hosts, int hosts_count, int count, int interval, int size, int timeout,
 		char *error, int max_error_len)
 {
@@ -196,24 +269,36 @@ static int	process_ping(ZBX_FPING_HOST *hosts, int hosts_count, int count, int i
 
 	if (0 != (fping_existence & FPING_EXISTS) && 0 != hosts_count)
 	{
-		if (FPING_UNINITIALIZED_INTERVAL == packet_interval)
-			packet_interval = get_interval_option(CONFIG_FPING_LOCATION, hosts[0].addr);
+		if (FPING_UNINITIALIZED_VALUE == packet_interval &&
+				SUCCEED != get_interval_option(CONFIG_FPING_LOCATION, hosts[0].addr, &packet_interval,
+				error, max_error_len))
+		{
+			return ret;
+		}
 
 		offset += zbx_snprintf(params + offset, sizeof(params) - offset, " -i%d", packet_interval);
 	}
 
 	if (0 != (fping_existence & FPING6_EXISTS) && 0 != hosts_count)
 	{
-		if (FPING_UNINITIALIZED_INTERVAL == packet_interval6)
-			packet_interval6 = get_interval_option(CONFIG_FPING6_LOCATION, hosts[0].addr);
+		if (FPING_UNINITIALIZED_VALUE == packet_interval6 &&
+				SUCCEED != get_interval_option(CONFIG_FPING6_LOCATION, hosts[0].addr, &packet_interval6,
+				error, max_error_len))
+		{
+			return ret;
+		}
 
 		offset6 += zbx_snprintf(params6 + offset6, sizeof(params6) - offset6, " -i%d", packet_interval6);
 	}
 #else
 	if (0 != hosts_count)
 	{
-		if (FPING_UNINITIALIZED_INTERVAL == packet_interval)
-			packet_interval = get_interval_option(CONFIG_FPING_LOCATION, hosts[0].addr);
+		if (FPING_UNINITIALIZED_VALUE == packet_interval &&
+				SUCCEED != get_interval_option(CONFIG_FPING_LOCATION, hosts[0].addr, &packet_interval,
+				error, max_error_len))
+		{
+			return ret;
+		}
 
 		offset += zbx_snprintf(params + offset, sizeof(params) - offset, " -i%d", packet_interval);
 	}
@@ -284,12 +369,19 @@ static int	process_ping(ZBX_FPING_HOST *hosts, int hosts_count, int count, int i
 		offset = 0;
 
 		if (0 != (fping_existence & FPING_EXISTS))
+		{
+			if (FPING_UNINITIALIZED_VALUE == fping_ipv6_supported)
+				fping_ipv6_supported = get_ipv6_support(CONFIG_FPING_LOCATION,hosts[0].addr);
+
 			offset += zbx_snprintf(tmp + offset, sizeof(tmp) - offset,
 					"%s %s 2>&1 <%s;", CONFIG_FPING_LOCATION, params, filename);
+		}
 
-		if (0 != (fping_existence & FPING6_EXISTS))
+		if (0 != (fping_existence & FPING6_EXISTS) && SUCCEED != fping_ipv6_supported)
+		{
 			zbx_snprintf(tmp + offset, sizeof(tmp) - offset,
 					"%s %s 2>&1 <%s;", CONFIG_FPING6_LOCATION, params6, filename);
+		}
 	}
 #else
 	zbx_snprintf(tmp, sizeof(tmp), "%s %s 2>&1 <%s", CONFIG_FPING_LOCATION, params, filename);
