@@ -23,6 +23,7 @@
 #include "log.h"
 #include "dbcache.h"
 #include "preproc.h"
+#include "daemon.h"
 
 #include "zbxserver.h"
 #include "zbxregexp.h"
@@ -676,6 +677,11 @@ static void	process_httptest(DC_HOST *host, zbx_httptest_t *httptest)
 	substitute_simple_macros(NULL, NULL, NULL, NULL, &host->hostid, NULL, NULL, NULL, NULL, &buffer,
 			MACRO_TYPE_COMMON, NULL, 0);
 
+	/* Avoid the potential usage of uninitialized values when: */
+	/* 1) compile without libCURL support */
+	/* 2) update interval is invalid */
+	db_httpstep.name = NULL;
+
 	if (SUCCEED != is_time_suffix(buffer, &delay, ZBX_LENGTH_UNLIMITED))
 	{
 		err_str = zbx_dsprintf(err_str, "update interval \"%s\" is invalid", buffer);
@@ -683,10 +689,6 @@ static void	process_httptest(DC_HOST *host, zbx_httptest_t *httptest)
 		goto httptest_error;
 	}
 
-	/* Explicitly initialize the name. If we compile without libCURL support, */
-	/* we avoid the potential usage of uninitialized values. */
-	db_httpstep.name = NULL;
-
 #ifdef HAVE_LIBCURL
 	if (NULL == (easyhandle = curl_easy_init()))
 	{
@@ -715,7 +717,7 @@ static void	process_httptest(DC_HOST *host, zbx_httptest_t *httptest)
 	httpstep.httptest = httptest;
 	httpstep.httpstep = &db_httpstep;
 
-	while (NULL != (row = DBfetch(result)))
+	while (NULL != (row = DBfetch(result)) && ZBX_IS_RUNNING())
 	{
 		struct curl_slist	*headers_slist = NULL;
 		char			*header_cookie = NULL;
@@ -733,6 +735,14 @@ static void	process_httptest(DC_HOST *host, zbx_httptest_t *httptest)
 				&db_httpstep.url, MACRO_TYPE_HTTPTEST_FIELD, NULL, 0);
 		http_substitute_variables(httptest, &db_httpstep.url);
 
+		db_httpstep.required = zbx_strdup(NULL, row[6]);
+		substitute_simple_macros(NULL, NULL, NULL, NULL, NULL, host, NULL, NULL, NULL,
+				&db_httpstep.required, MACRO_TYPE_HTTPTEST_FIELD, NULL, 0);
+
+		db_httpstep.status_codes = zbx_strdup(NULL, row[7]);
+		substitute_simple_macros(NULL, NULL, NULL, NULL, &host->hostid, NULL, NULL, NULL, NULL,
+				&db_httpstep.status_codes, MACRO_TYPE_COMMON, NULL, 0);
+
 		db_httpstep.post_type = atoi(row[8]);
 
 		if (ZBX_POSTTYPE_RAW == db_httpstep.post_type)
@@ -766,14 +776,6 @@ static void	process_httptest(DC_HOST *host, zbx_httptest_t *httptest)
 			goto httpstep_error;
 		}
 
-		db_httpstep.required = zbx_strdup(NULL, row[6]);
-		substitute_simple_macros(NULL, NULL, NULL, NULL, NULL, host, NULL, NULL, NULL,
-				&db_httpstep.required, MACRO_TYPE_HTTPTEST_FIELD, NULL, 0);
-
-		db_httpstep.status_codes = zbx_strdup(NULL, row[7]);
-		substitute_simple_macros(NULL, NULL, NULL, NULL, &host->hostid, NULL, NULL, NULL, NULL,
-				&db_httpstep.status_codes, MACRO_TYPE_COMMON, NULL, 0);
-
 		db_httpstep.follow_redirects = atoi(row[9]);
 		db_httpstep.retrieve_mode = atoi(row[10]);
 
@@ -859,6 +861,7 @@ static void	process_httptest(DC_HOST *host, zbx_httptest_t *httptest)
 		do
 		{
 			memset(&page, 0, sizeof(page));
+			errbuf[0] = '\0';
 
 			if (CURLE_OK == (err = curl_easy_perform(easyhandle)))
 				break;
@@ -1089,7 +1092,7 @@ int	process_httptests(int httppoller_num, int now)
 			HOST_STATUS_MONITORED,
 			HOST_MAINTENANCE_STATUS_OFF, MAINTENANCE_TYPE_NORMAL);
 
-	while (NULL != (row = DBfetch(result)))
+	while (NULL != (row = DBfetch(result)) && ZBX_IS_RUNNING())
 	{
 		ZBX_STR2UINT64(host.hostid, row[0]);
 		strscpy(host.host, row[1]);
