@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -43,45 +43,58 @@ static zbx_uint64_t	select_discovered_host(const DB_EVENT *event)
 	const char	*__function_name = "select_discovered_host";
 	DB_RESULT	result;
 	DB_ROW		row;
-	zbx_uint64_t	hostid = 0;
-	char		*sql = NULL;
+	zbx_uint64_t	hostid = 0, proxy_hostid;
+	char		*sql = NULL, *ip_esc;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s() eventid:" ZBX_FS_UI64,
-			__function_name, event->eventid);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s() eventid:" ZBX_FS_UI64, __function_name, event->eventid);
 
 	switch (event->object)
 	{
 		case EVENT_OBJECT_DHOST:
-			sql = zbx_dsprintf(sql,
-				"select h.hostid,h.status"
-				" from hosts h,interface i,dservices ds,dchecks dc,drules dr"
-				" where h.hostid=i.hostid"
-					" and i.ip=ds.ip"
-					" and ds.dcheckid=dc.dcheckid"
-					" and dc.druleid=dr.druleid"
-					" and h.status in (%d,%d)"
-					" and " ZBX_SQL_NULLCMP("dr.proxy_hostid", "h.proxy_hostid")
-					" and i.useip=1"
-					" and ds.dhostid=" ZBX_FS_UI64
-				" order by i.hostid",
-				HOST_STATUS_MONITORED, HOST_STATUS_NOT_MONITORED,
-				event->objectid);
-			break;
 		case EVENT_OBJECT_DSERVICE:
+			result = DBselect(
+					"select dr.proxy_hostid,ds.ip"
+					" from drules dr,dchecks dc,dservices ds"
+					" where dc.druleid=dr.druleid"
+						" and ds.dcheckid=dc.dcheckid"
+						" and ds.%s=" ZBX_FS_UI64,
+					EVENT_OBJECT_DSERVICE == event->object ? "dserviceid" : "dhostid",
+					event->objectid);
+
+			if (NULL == (row = DBfetch(result)))
+			{
+				DBfree_result(result);
+				goto exit;
+			}
+
+			ZBX_DBROW2UINT64(proxy_hostid, row[0]);
+			ip_esc = DBdyn_escape_string(row[1]);
+			DBfree_result(result);
+
 			sql = zbx_dsprintf(sql,
-				"select h.hostid,h.status"
-				" from hosts h,interface i,dservices ds,dchecks dc,drules dr"
-				" where h.hostid=i.hostid"
-					" and i.ip=ds.ip"
-					" and ds.dcheckid=dc.dcheckid"
-					" and dc.druleid=dr.druleid"
-					" and h.status in (%d,%d)"
-					" and " ZBX_SQL_NULLCMP("dr.proxy_hostid", "h.proxy_hostid")
-					" and i.useip=1"
-					" and ds.dserviceid=" ZBX_FS_UI64
-				" order by i.hostid",
-				HOST_STATUS_MONITORED, HOST_STATUS_NOT_MONITORED,
-				event->objectid);
+					"select h.hostid"
+					" from hosts h,interface i"
+					" where h.hostid=i.hostid"
+						" and i.ip='%s'"
+						" and i.useip=1"
+						" and h.status in (%d,%d)"
+						" and h.proxy_hostid%s"
+					" order by i.hostid",
+					ip_esc,
+					HOST_STATUS_MONITORED, HOST_STATUS_NOT_MONITORED,
+					DBsql_id_cmp(proxy_hostid));
+
+			zbx_free(ip_esc);
+			break;
+		case EVENT_OBJECT_ZABBIX_ACTIVE:
+			sql = zbx_dsprintf(sql,
+					"select h.hostid"
+					" from hosts h,autoreg_host a"
+					" where h.host=a.host"
+						" and a.autoreg_hostid=" ZBX_FS_UI64
+						" and h.status in (%d,%d)",
+					event->objectid,
+					HOST_STATUS_MONITORED, HOST_STATUS_NOT_MONITORED);
 			break;
 		default:
 			goto exit;
@@ -122,7 +135,7 @@ static void	add_discovered_host_groups(zbx_uint64_t hostid, zbx_vector_uint64_t
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
-	sql = zbx_malloc(sql, sql_alloc);
+	sql = (char *)zbx_malloc(sql, sql_alloc);
 
 	zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset,
 			"select groupid"
@@ -413,6 +426,30 @@ clean:
 	return hostid;
 }
 
+/******************************************************************************
+ *                                                                            *
+ * Function: is_discovery_or_auto_registration                                *
+ *                                                                            *
+ * Purpose: checks if the event is discovery or auto registration event       *
+ *                                                                            *
+ * Return value: SUCCEED - it's discovery or auto registration event          *
+ *               FAIL    - otherwise                                          *
+ *                                                                            *
+ ******************************************************************************/
+static int	is_discovery_or_auto_registration(const DB_EVENT *event)
+{
+	if (event->source == EVENT_SOURCE_DISCOVERY && (event->object == EVENT_OBJECT_DHOST ||
+			event->object == EVENT_OBJECT_DSERVICE))
+	{
+		return SUCCEED;
+	}
+
+	if (event->source == EVENT_SOURCE_AUTO_REGISTRATION && event->object == EVENT_OBJECT_ZABBIX_ACTIVE)
+		return SUCCEED;
+
+	return FAIL;
+}
+
 /******************************************************************************
  *                                                                            *
  * Function: op_host_add                                                      *
@@ -431,10 +468,7 @@ void	op_host_add(const DB_EVENT *event)
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
-	if (event->source != EVENT_SOURCE_DISCOVERY && event->source != EVENT_SOURCE_AUTO_REGISTRATION)
-		return;
-
-	if (event->object != EVENT_OBJECT_DHOST && event->object != EVENT_OBJECT_DSERVICE && event->object != EVENT_OBJECT_ZABBIX_ACTIVE)
+	if (FAIL == is_discovery_or_auto_registration(event))
 		return;
 
 	add_discovered_host(event);
@@ -460,10 +494,7 @@ void	op_host_del(const DB_EVENT *event)
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
-	if (event->source != EVENT_SOURCE_DISCOVERY)
-		return;
-
-	if (event->object != EVENT_OBJECT_DHOST && event->object != EVENT_OBJECT_DSERVICE)
+	if (FAIL == is_discovery_or_auto_registration(event))
 		return;
 
 	if (0 == (hostid = select_discovered_host(event)))
@@ -496,10 +527,7 @@ void	op_host_enable(const DB_EVENT *event)
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
-	if (event->source != EVENT_SOURCE_DISCOVERY)
-		return;
-
-	if (event->object != EVENT_OBJECT_DHOST && event->object != EVENT_OBJECT_DSERVICE)
+	if (FAIL == is_discovery_or_auto_registration(event))
 		return;
 
 	if (0 == (hostid = add_discovered_host(event)))
@@ -531,10 +559,7 @@ void	op_host_disable(const DB_EVENT *event)
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
-	if (event->source != EVENT_SOURCE_DISCOVERY && event->source != EVENT_SOURCE_AUTO_REGISTRATION)
-		return;
-
-	if (event->object != EVENT_OBJECT_DHOST && event->object != EVENT_OBJECT_DSERVICE && event->object != EVENT_OBJECT_ZABBIX_ACTIVE)
+	if (FAIL == is_discovery_or_auto_registration(event))
 		return;
 
 	if (0 == (hostid = add_discovered_host(event)))
@@ -571,15 +596,9 @@ void	op_host_inventory_mode(const DB_EVENT *event, int inventory_mode)
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
-	if (event->source != EVENT_SOURCE_DISCOVERY && event->source != EVENT_SOURCE_AUTO_REGISTRATION)
+	if (FAIL == is_discovery_or_auto_registration(event))
 		return;
 
-	if (event->object != EVENT_OBJECT_DHOST && event->object != EVENT_OBJECT_DSERVICE &&
-			event->object != EVENT_OBJECT_ZABBIX_ACTIVE)
-	{
-		return;
-	}
-
 	if (0 == (hostid = add_discovered_host(event)))
 		return;
 
@@ -607,10 +626,7 @@ void	op_groups_add(const DB_EVENT *event, zbx_vector_uint64_t *groupids)
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
-	if (event->source != EVENT_SOURCE_DISCOVERY && event->source != EVENT_SOURCE_AUTO_REGISTRATION)
-		return;
-
-	if (event->object != EVENT_OBJECT_DHOST && event->object != EVENT_OBJECT_DSERVICE && event->object != EVENT_OBJECT_ZABBIX_ACTIVE)
+	if (FAIL == is_discovery_or_auto_registration(event))
 		return;
 
 	if (0 == (hostid = add_discovered_host(event)))
@@ -644,16 +660,13 @@ void	op_groups_del(const DB_EVENT *event, zbx_vector_uint64_t *groupids)
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
-	if (event->source != EVENT_SOURCE_DISCOVERY)
-		return;
-
-	if (event->object != EVENT_OBJECT_DHOST && event->object != EVENT_OBJECT_DSERVICE)
+	if (FAIL == is_discovery_or_auto_registration(event))
 		return;
 
 	if (0 == (hostid = select_discovered_host(event)))
 		return;
 
-	sql = zbx_malloc(sql, sql_alloc);
+	sql = (char *)zbx_malloc(sql, sql_alloc);
 
 	/* make sure host belongs to at least one hostgroup */
 	zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset,
@@ -710,10 +723,7 @@ void	op_template_add(const DB_EVENT *event, zbx_vector_uint64_t *lnk_templateids
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
-	if (event->source != EVENT_SOURCE_DISCOVERY && event->source != EVENT_SOURCE_AUTO_REGISTRATION)
-		return;
-
-	if (event->object != EVENT_OBJECT_DHOST && event->object != EVENT_OBJECT_DSERVICE && event->object != EVENT_OBJECT_ZABBIX_ACTIVE)
+	if (FAIL == is_discovery_or_auto_registration(event))
 		return;
 
 	if (0 == (hostid = add_discovered_host(event)))
@@ -748,10 +758,7 @@ void	op_template_del(const DB_EVENT *event, zbx_vector_uint64_t *del_templateids
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
-	if (event->source != EVENT_SOURCE_DISCOVERY)
-		return;
-
-	if (event->object != EVENT_OBJECT_DHOST && event->object != EVENT_OBJECT_DSERVICE)
+	if (FAIL == is_discovery_or_auto_registration(event))
 		return;
 
 	if (0 == (hostid = select_discovered_host(event)))
