@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -29,6 +29,41 @@
 
 extern unsigned char	program_type;
 
+/******************************************************************************
+ *                                                                            *
+ * Function: db_register_host                                                 *
+ *                                                                            *
+ * Purpose: perform active agent auto registration                            *
+ *                                                                            *
+ * Parameters: host          - [IN] name of the host to be added or updated   *
+ *             ip            - [IN] IP address of the host                    *
+ *             port          - [IN] port of the host                          *
+ *             host_metadata - [IN] host metadata                             *
+ *                                                                            *
+ * Comments: helper function for get_hostid_by_host                           *
+ *                                                                            *
+ ******************************************************************************/
+static void	db_register_host(const char *host, const char *ip, unsigned short port, const char *host_metadata)
+{
+	char	dns[INTERFACE_DNS_LEN_MAX];
+
+	if (0 == strncmp("::ffff:", ip, 7) && SUCCEED == is_ip4(ip + 7))
+		ip += 7;
+
+	zbx_alarm_on(CONFIG_TIMEOUT);
+	zbx_gethost_by_ip(ip, dns, sizeof(dns));
+	zbx_alarm_off();
+
+	DBbegin();
+
+	if (0 != (program_type & ZBX_PROGRAM_TYPE_SERVER))
+		DBregister_host(0, host, ip, dns, port, host_metadata, (int)time(NULL));
+	else if (0 != (program_type & ZBX_PROGRAM_TYPE_PROXY))
+		DBproxy_register_host(host, ip, dns, port, host_metadata);
+
+	DBcommit();
+}
+
 /******************************************************************************
  *                                                                            *
  * Function: get_hostid_by_host                                               *
@@ -42,7 +77,8 @@ extern unsigned char	program_type;
  *                                                                            *
  * Author: Alexander Vladishev                                                *
  *                                                                            *
- * Comments: NB! adds host to the database if it does not exist               *
+ * Comments: NB! adds host to the database if it does not exist or if it      *
+ *           exists but metadata has changed                                  *
  *                                                                            *
  ******************************************************************************/
 static int	get_hostid_by_host(const zbx_socket_t *sock, const char *host, const char *ip, unsigned short port,
@@ -50,12 +86,12 @@ static int	get_hostid_by_host(const zbx_socket_t *sock, const char *host, const
 {
 	const char	*__function_name = "get_hostid_by_host";
 
-	char		*host_esc, dns[INTERFACE_DNS_LEN_MAX], *ch_error;
+	char		*host_esc, *ch_error, *old_metadata;
 	DB_RESULT	result;
 	DB_ROW		row;
 	int		ret = FAIL;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s() host:'%s'", __function_name, host);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s() host:'%s' metadata:'%s'", __function_name, host, host_metadata);
 
 	if (FAIL == zbx_check_hostname(host, &ch_error))
 	{
@@ -69,116 +105,112 @@ static int	get_hostid_by_host(const zbx_socket_t *sock, const char *host, const
 	result =
 #if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
 		DBselect(
-			"select hostid,status,tls_accept,tls_issuer,tls_subject,tls_psk_identity"
-			" from hosts"
-			" where host='%s'"
-				" and status in (%d,%d)"
-				" and flags<>%d"
-				" and proxy_hostid is null",
+			"select h.hostid,h.status,h.tls_accept,h.tls_issuer,h.tls_subject,h.tls_psk_identity,"
+			"a.host_metadata"
+			" from hosts h"
+				" left join autoreg_host a"
+					" on a.proxy_hostid is null and a.host=h.host"
+			" where h.host='%s'"
+				" and h.status in (%d,%d)"
+				" and h.flags<>%d"
+				" and h.proxy_hostid is null",
 			host_esc, HOST_STATUS_MONITORED, HOST_STATUS_NOT_MONITORED, ZBX_FLAG_DISCOVERY_PROTOTYPE);
 #else
 		DBselect(
-			"select hostid,status,tls_accept"
-			" from hosts"
-			" where host='%s'"
-				" and status in (%d,%d)"
-				" and flags<>%d"
-				" and proxy_hostid is null",
+			"select h.hostid,h.status,h.tls_accept,a.host_metadata"
+			" from hosts h"
+				" left join autoreg_host a"
+					" on a.proxy_hostid is null and a.host=h.host"
+			" where h.host='%s'"
+				" and h.status in (%d,%d)"
+				" and h.flags<>%d"
+				" and h.proxy_hostid is null",
 			host_esc, HOST_STATUS_MONITORED, HOST_STATUS_NOT_MONITORED, ZBX_FLAG_DISCOVERY_PROTOTYPE);
 #endif
 	if (NULL != (row = DBfetch(result)))
 	{
-		if (HOST_STATUS_MONITORED == atoi(row[1]))
+		if (0 == ((unsigned int)atoi(row[2]) & sock->connection_type))
 		{
-			unsigned int	tls_accept;
+			zbx_snprintf(error, MAX_STRING_LEN, "connection of type \"%s\" is not allowed for host"
+					" \"%s\"", zbx_tcp_connection_type_name(sock->connection_type), host);
+			goto done;
+		}
 
-			tls_accept = (unsigned int)atoi(row[2]);
+#if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
+		if (ZBX_TCP_SEC_TLS_CERT == sock->connection_type)
+		{
+			zbx_tls_conn_attr_t	attr;
 
-			if (0 == (tls_accept & sock->connection_type))
+			if (SUCCEED != zbx_tls_get_attr_cert(sock, &attr))
 			{
-				zbx_snprintf(error, MAX_STRING_LEN, "connection of type \"%s\" is not allowed for host"
-						" \"%s\"", zbx_tcp_connection_type_name(sock->connection_type), host);
+				THIS_SHOULD_NEVER_HAPPEN;
+
+				zbx_snprintf(error, MAX_STRING_LEN, "cannot get connection attributes for host"
+						" \"%s\"", host);
 				goto done;
 			}
 
-#if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
-			if (ZBX_TCP_SEC_TLS_CERT == sock->connection_type)
+			/* simplified match, not compliant with RFC 4517, 4518 */
+			if ('\0' != *row[3] && 0 != strcmp(row[3], attr.issuer))
 			{
-				zbx_tls_conn_attr_t	attr;
-
-				if (SUCCEED != zbx_tls_get_attr_cert(sock, &attr))
-				{
-					THIS_SHOULD_NEVER_HAPPEN;
-
-					zbx_snprintf(error, MAX_STRING_LEN, "cannot get connection attributes for host"
-							" \"%s\"", host);
-					goto done;
-				}
-
-				/* simplified match, not compliant with RFC 4517, 4518 */
-				if ('\0' != *row[3] && 0 != strcmp(row[3], attr.issuer))
-				{
-					zbx_snprintf(error, MAX_STRING_LEN, "certificate issuer does not match for"
-							" host \"%s\"", host);
-					goto done;
-				}
-
-				/* simplified match, not compliant with RFC 4517, 4518 */
-				if ('\0' != *row[4] && 0 != strcmp(row[4], attr.subject))
-				{
-					zbx_snprintf(error, MAX_STRING_LEN, "certificate subject does not match for"
-							" host \"%s\"", host);
-					goto done;
-				}
+				zbx_snprintf(error, MAX_STRING_LEN, "certificate issuer does not match for"
+						" host \"%s\"", host);
+				goto done;
 			}
-			else if (ZBX_TCP_SEC_TLS_PSK == sock->connection_type)
+
+			/* simplified match, not compliant with RFC 4517, 4518 */
+			if ('\0' != *row[4] && 0 != strcmp(row[4], attr.subject))
 			{
-				zbx_tls_conn_attr_t	attr;
-
-				if (SUCCEED != zbx_tls_get_attr_psk(sock, &attr))
-				{
-					THIS_SHOULD_NEVER_HAPPEN;
-
-					zbx_snprintf(error, MAX_STRING_LEN, "cannot get connection attributes for host"
-							" \"%s\"", host);
-					goto done;
-				}
-
-				if (strlen(row[5]) != attr.psk_identity_len ||
-						0 != memcmp(row[5], attr.psk_identity, attr.psk_identity_len))
-				{
-					zbx_snprintf(error, MAX_STRING_LEN, "false PSK identity for host \"%s\"", host);
-					goto done;
-				}
+				zbx_snprintf(error, MAX_STRING_LEN, "certificate subject does not match for"
+						" host \"%s\"", host);
+				goto done;
 			}
-#endif
-			ZBX_STR2UINT64(*hostid, row[0]);
-
-			ret = SUCCEED;
 		}
-		else
+#if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || (defined(HAVE_OPENSSL) && defined(HAVE_OPENSSL_WITH_PSK))
+		else if (ZBX_TCP_SEC_TLS_PSK == sock->connection_type)
+		{
+			zbx_tls_conn_attr_t	attr;
+
+			if (SUCCEED != zbx_tls_get_attr_psk(sock, &attr))
+			{
+				THIS_SHOULD_NEVER_HAPPEN;
+
+				zbx_snprintf(error, MAX_STRING_LEN, "cannot get connection attributes for host"
+						" \"%s\"", host);
+				goto done;
+			}
+
+			if (strlen(row[5]) != attr.psk_identity_len ||
+					0 != memcmp(row[5], attr.psk_identity, attr.psk_identity_len))
+			{
+				zbx_snprintf(error, MAX_STRING_LEN, "false PSK identity for host \"%s\"", host);
+				goto done;
+			}
+		}
+#endif
+		old_metadata = row[6];
+#else
+		old_metadata = row[3];
+#endif
+		/* metadata is available only on Zabbix server */
+		if (SUCCEED == DBis_null(old_metadata) || 0 != strcmp(old_metadata, host_metadata))
+		{
+			db_register_host(host, ip, port, host_metadata);
+		}
+
+		if (HOST_STATUS_MONITORED != atoi(row[1]))
+		{
 			zbx_snprintf(error, MAX_STRING_LEN, "host [%s] not monitored", host);
+			goto done;
+		}
+
+		ZBX_STR2UINT64(*hostid, row[0]);
+		ret = SUCCEED;
 	}
 	else
 	{
 		zbx_snprintf(error, MAX_STRING_LEN, "host [%s] not found", host);
-
-		/* remove ::ffff: prefix from IPv4-mapped IPv6 addresses */
-		if (0 == strncmp("::ffff:", ip, 7) && SUCCEED == is_ip4(ip + 7))
-			ip += 7;
-
-		zbx_alarm_on(CONFIG_TIMEOUT);
-		zbx_gethost_by_ip(ip, dns, sizeof(dns));
-		zbx_alarm_off();
-
-		DBbegin();
-
-		if (0 != (program_type & ZBX_PROGRAM_TYPE_SERVER))
-			DBregister_host(0, host, ip, dns, port, host_metadata, (int)time(NULL));
-		else if (0 != (program_type & ZBX_PROGRAM_TYPE_PROXY))
-			DBproxy_register_host(host, ip, dns, port, host_metadata);
-
-		DBcommit();
+		db_register_host(host, ip, port, host_metadata);
 	}
 done:
 	DBfree_result(result);
@@ -260,7 +292,7 @@ int	send_list_of_active_checks(zbx_socket_t *sock, char *request)
 
 	get_list_of_active_checks(hostid, &itemids);
 
-	buffer = zbx_malloc(buffer, buffer_alloc);
+	buffer = (char *)zbx_malloc(buffer, buffer_alloc);
 
 	if (0 != itemids.values_num)
 	{
@@ -268,8 +300,8 @@ int	send_list_of_active_checks(zbx_socket_t *sock, char *request)
 		int		*errcodes, now;
 		zbx_config_t	cfg;
 
-		dc_items = zbx_malloc(NULL, sizeof(DC_ITEM) * itemids.values_num);
-		errcodes = zbx_malloc(NULL, sizeof(int) * itemids.values_num);
+		dc_items = (DC_ITEM *)zbx_malloc(NULL, sizeof(DC_ITEM) * itemids.values_num);
+		errcodes = (int *)zbx_malloc(NULL, sizeof(int) * itemids.values_num);
 
 		DCconfig_get_items_by_itemids(dc_items, itemids.values, errcodes, itemids.values_num);
 		zbx_config_get(&cfg, ZBX_CONFIG_FLAGS_REFRESH_UNSUPPORTED);
@@ -278,6 +310,8 @@ int	send_list_of_active_checks(zbx_socket_t *sock, char *request)
 
 		for (i = 0; i < itemids.values_num; i++)
 		{
+			int	delay;
+
 			if (SUCCEED != errcodes[i])
 			{
 				zabbix_log(LOG_LEVEL_DEBUG, "%s() Item [" ZBX_FS_UI64 "] was not found in the"
@@ -300,8 +334,11 @@ int	send_list_of_active_checks(zbx_socket_t *sock, char *request)
 					continue;
 			}
 
+			if (SUCCEED != zbx_interval_preproc(dc_items[i].delay, &delay, NULL, NULL))
+				continue;
+
 			zbx_snprintf_alloc(&buffer, &buffer_alloc, &buffer_offset, "%s:%d:" ZBX_FS_UI64 "\n",
-					dc_items[i].key_orig, dc_items[i].delay, dc_items[i].lastlogsize);
+					dc_items[i].key_orig, delay, dc_items[i].lastlogsize);
 		}
 
 		zbx_config_clean(&cfg);
@@ -443,21 +480,21 @@ int	send_list_of_active_checks_json(zbx_socket_t *sock, struct zbx_json_parse *j
 	zbx_vector_ptr_create(&regexps);
 	zbx_vector_str_create(&names);
 
-	if (FAIL == zbx_json_value_by_name(jp, ZBX_PROTO_TAG_HOST, host, sizeof(host)))
+	if (FAIL == zbx_json_value_by_name(jp, ZBX_PROTO_TAG_HOST, host, sizeof(host), NULL))
 	{
 		zbx_snprintf(error, MAX_STRING_LEN, "%s", zbx_json_strerror());
 		goto error;
 	}
 
-	host_metadata = zbx_malloc(host_metadata, host_metadata_alloc);
+	host_metadata = (char *)zbx_malloc(host_metadata, host_metadata_alloc);
 
 	if (FAIL == zbx_json_value_by_name_dyn(jp, ZBX_PROTO_TAG_HOST_METADATA,
-			&host_metadata, &host_metadata_alloc))
+			&host_metadata, &host_metadata_alloc, NULL))
 	{
 		*host_metadata = '\0';
 	}
 
-	if (FAIL == zbx_json_value_by_name(jp, ZBX_PROTO_TAG_IP, ip, sizeof(ip)))
+	if (FAIL == zbx_json_value_by_name(jp, ZBX_PROTO_TAG_IP, ip, sizeof(ip), NULL))
 		strscpy(ip, sock->peer);
 
 	if (FAIL == is_ip(ip))	/* check even if 'ip' came from get_ip_by_socket() - it can return not a valid IP */
@@ -466,7 +503,7 @@ int	send_list_of_active_checks_json(zbx_socket_t *sock, struct zbx_json_parse *j
 		goto error;
 	}
 
-	if (FAIL == zbx_json_value_by_name(jp, ZBX_PROTO_TAG_PORT, tmp, sizeof(tmp)))
+	if (FAIL == zbx_json_value_by_name(jp, ZBX_PROTO_TAG_PORT, tmp, sizeof(tmp), NULL))
 	{
 		port = ZBX_DEFAULT_AGENT_PORT;
 	}
@@ -493,8 +530,8 @@ int	send_list_of_active_checks_json(zbx_socket_t *sock, struct zbx_json_parse *j
 		int		*errcodes, now, delay;
 		zbx_config_t	cfg;
 
-		dc_items = zbx_malloc(NULL, sizeof(DC_ITEM) * itemids.values_num);
-		errcodes = zbx_malloc(NULL, sizeof(int) * itemids.values_num);
+		dc_items = (DC_ITEM *)zbx_malloc(NULL, sizeof(DC_ITEM) * itemids.values_num);
+		errcodes = (int *)zbx_malloc(NULL, sizeof(int) * itemids.values_num);
 
 		DCconfig_get_items_by_itemids(dc_items, itemids.values, errcodes, itemids.values_num);
 		zbx_config_get(&cfg, ZBX_CONFIG_FLAGS_REFRESH_UNSUPPORTED);
@@ -572,7 +609,7 @@ int	send_list_of_active_checks_json(zbx_socket_t *sock, struct zbx_json_parse *j
 
 		for (i = 0; i < regexps.values_num; i++)
 		{
-			zbx_expression_t	*regexp = regexps.values[i];
+			zbx_expression_t	*regexp = (zbx_expression_t *)regexps.values[i];
 
 			zbx_json_addobject(&json, NULL);
 			zbx_json_addstring(&json, "name", regexp->name, ZBX_JSON_TYPE_STRING);
